=== modified file 'breezy/bzr/inventorytree.py'
--- breezy/bzr/inventorytree.py	2020-08-10 15:00:17 +0000
+++ breezy/bzr/inventorytree.py	2020-08-22 22:46:24 +0000
@@ -104,6 +104,14 @@
     def is_reparented(self):
         return self.parent_id[0] != self.parent_id[1]
 
+    @property
+    def renamed(self):
+        return (
+            not self.copied and
+            None not in self.name and
+            None not in self.parent_id and
+            (self.name[0] != self.name[1] or self.parent_id[0] != self.parent_id[1]))
+
     def discard_new(self):
         return self.__class__(
             self.file_id, (self.path[0], None), self.changed_content,
@@ -1010,8 +1018,7 @@
             subdirs = []
             for name, child in entry.sorted_children():
                 toppath = relroot + name
-                dirblock.append((toppath, name, child.kind, None,
-                                 child.kind))
+                dirblock.append((toppath, name, child.kind, None, child.kind))
                 if child.kind == _directory:
                     subdirs.append((toppath, child.file_id))
             yield root, dirblock

=== modified file 'breezy/bzr/transform.py'
--- breezy/bzr/transform.py	2020-08-10 15:00:17 +0000
+++ breezy/bzr/transform.py	2020-08-22 22:46:24 +0000
@@ -110,7 +110,7 @@
 
     def create_path(self, name, parent):
         """Assign a transaction id to a new path"""
-        trans_id = self._assign_id()
+        trans_id = self.assign_id()
         unique_add(self._new_name, trans_id, name)
         unique_add(self._new_parent, trans_id, parent)
         return trans_id
@@ -225,7 +225,7 @@
                 if file_id in self._non_present_ids:
                     return self._non_present_ids[file_id]
                 else:
-                    trans_id = self._assign_id()
+                    trans_id = self.assign_id()
                     self._non_present_ids[file_id] = trans_id
                     return trans_id
             else:
@@ -1912,6 +1912,9 @@
         self._iter_changes_cache = {
             c.file_id: c for c in self._transform.iter_changes()}
 
+    def supports_setting_file_ids(self):
+        return True
+
     def supports_tree_reference(self):
         # TODO(jelmer): Support tree references in PreviewTree.
         # return self._transform._tree.supports_tree_reference()
@@ -2124,6 +2127,7 @@
             limbo_name = tt._limbo_name(trans_id)
             if trans_id in tt._new_reference_revision:
                 kind = 'tree-reference'
+                link_or_sha1 = tt._new_reference_revision
             if kind == 'file':
                 statval = os.lstat(limbo_name)
                 size = statval.st_size
@@ -2167,21 +2171,28 @@
         file_id = self.path2id(path)
         changes = self._iter_changes_cache.get(file_id)
         if changes is None:
-            get_old = True
+            if file_id is None:
+                old_path = None
+            else:
+                old_path = self._transform._tree.id2path(file_id)
         else:
-            changed_content, versioned, kind = (
-                changes.changed_content, changes.versioned, changes.kind)
-            if kind[1] is None:
+            if changes.kind[1] is None:
                 return None
-            get_old = (kind[0] == 'file' and versioned[0])
-        if get_old:
+            if changes.kind[0] == 'file' and changes.versioned[0]:
+                old_path = changes.path[0]
+            else:
+                old_path = None
+        if old_path is not None:
             old_annotation = self._transform._tree.annotate_iter(
-                path, default_revision=default_revision)
+                old_path, default_revision=default_revision)
         else:
             old_annotation = []
         if changes is None:
-            return old_annotation
-        if not changed_content:
+            if old_path is None:
+                return None
+            else:
+                return old_annotation
+        if not changes.changed_content:
             return old_annotation
         # TODO: This is doing something similar to what WT.annotate_iter is
         #       doing, however it fails slightly because it doesn't know what
@@ -2191,7 +2202,7 @@
         #       It would be nice to be able to use the new Annotator based
         #       approach, as well.
         return annotate.reannotate([old_annotation],
-                                   self.get_file(path).readlines(),
+                                   self.get_file_lines(path),
                                    default_revision)
 
     def walkdirs(self, prefix=''):
@@ -2213,7 +2224,7 @@
                 else:
                     kind = 'unknown'
                     versioned_kind = self._transform._tree.stored_kind(
-                        self._transform._tree.id2path(file_id))
+                        path_from_root)
                 if versioned_kind == 'directory':
                     subdirs.append(child_id)
                 children.append((path_from_root, basename, kind, None,

=== modified file 'breezy/bzr/workingtree_4.py'
--- breezy/bzr/workingtree_4.py	2020-08-10 15:00:17 +0000
+++ breezy/bzr/workingtree_4.py	2020-08-22 22:46:24 +0000
@@ -465,7 +465,13 @@
 
     def get_reference_revision(self, path):
         # referenced tree's revision is whatever's currently there
-        return self.get_nested_tree(path).last_revision()
+        try:
+            return self.get_nested_tree(path).last_revision()
+        except errors.NotBranchError:
+            entry = self._get_entry(path=path)
+            if entry == (None, None):
+                return False
+            return entry[1][0][1]
 
     def get_nested_tree(self, path):
         return WorkingTree.open(self.abspath(path))

=== modified file 'breezy/conflicts.py'
--- breezy/conflicts.py	2020-08-10 15:00:17 +0000
+++ breezy/conflicts.py	2020-08-22 22:46:24 +0000
@@ -339,4 +339,5 @@
         raise NotImplementedError(self.do)
 
     def describe(self):
+        """Return a string description of this conflict."""
         raise NotImplementedError(self.describe)

=== modified file 'breezy/delta.py'
--- breezy/delta.py	2020-08-10 15:00:17 +0000
+++ breezy/delta.py	2020-08-22 22:46:24 +0000
@@ -327,14 +327,12 @@
         exe_change = False
         # files are "renamed" if they are moved or if name changes, as long
         # as it had a value
-        if None not in change.name and None not in change.parent_id and\
-                (change.name[0] != change.name[1] or change.parent_id[0] != change.parent_id[1]):
-            if change.copied:
-                copied = True
-                renamed = False
-            else:
-                renamed = True
-                copied = False
+        if change.copied:
+            copied = True
+            renamed = False
+        elif change.renamed:
+            renamed = True
+            copied = False
         else:
             copied = False
             renamed = False

=== modified file 'breezy/git/commit.py'
--- breezy/git/commit.py	2020-08-10 15:00:17 +0000
+++ breezy/git/commit.py	2020-08-22 22:46:24 +0000
@@ -19,6 +19,7 @@
 
 from dulwich.index import (
     commit_tree,
+    read_submodule_head,
     )
 import stat
 
@@ -42,7 +43,6 @@
     Blob,
     Commit,
     )
-from dulwich.index import read_submodule_head
 
 
 from .mapping import (
@@ -75,6 +75,9 @@
     def record_iter_changes(self, workingtree, basis_revid, iter_changes):
         seen_root = False
         for change in iter_changes:
+            if change.kind == (None, None):
+                # Ephemeral
+                continue
             if change.versioned[0] and not change.copied:
                 file_id = self._mapping.generate_file_id(change.path[0])
             elif change.versioned[1]:
@@ -115,7 +118,10 @@
                 finally:
                     f.close()
                 sha = blob.id
-                entry.text_size = st.st_size
+                if st is not None:
+                    entry.text_size = st.st_size
+                else:
+                    entry.text_size = len(blob.data)
                 entry.git_sha1 = sha
                 self.store.add_object(blob)
             elif change.kind[1] == "symlink":

=== modified file 'breezy/git/mapping.py'
--- breezy/git/mapping.py	2020-07-18 23:14:00 +0000
+++ breezy/git/mapping.py	2020-08-22 22:46:24 +0000
@@ -160,7 +160,7 @@
     """Class that maps between Git and Bazaar semantics."""
     experimental = False
 
-    BZR_DUMMY_FILE = None
+    BZR_DUMMY_FILE = None  # type: Optional[str]
 
     def is_special_file(self, filename):
         return (filename in (self.BZR_DUMMY_FILE, ))

=== modified file 'breezy/git/transform.py'
--- breezy/git/transform.py	2020-08-10 15:00:17 +0000
+++ breezy/git/transform.py	2020-08-22 22:46:24 +0000
@@ -19,13 +19,29 @@
 
 import errno
 import os
-from stat import S_ISREG
+import posixpath
+from stat import S_IEXEC, S_ISREG
 import time
 
-from .. import conflicts, errors, multiparent, osutils, trace, ui, urlutils
+from .mapping import encode_git_path, mode_kind, mode_is_executable, object_mode
+from .tree import GitTree, GitTreeDirectory, GitTreeSymlink, GitTreeFile
+
+from .. import (
+    annotate,
+    conflicts,
+    errors,
+    multiparent,
+    osutils,
+    revision as _mod_revision,
+    trace,
+    ui,
+    urlutils,
+    )
 from ..i18n import gettext
 from ..mutabletree import MutableTree
+from ..tree import InterTree, TreeChange
 from ..transform import (
+    PreviewTree,
     TreeTransform,
     _TransformResults,
     _FileMover,
@@ -38,10 +54,9 @@
     ReusingTransform,
     MalformedTransform,
     )
-from ..bzr.inventorytree import InventoryTreeChange
 
-from ..bzr import inventory
-from ..bzr.transform import TransformPreview as GitTransformPreview
+from dulwich.index import commit_tree, blob_from_path_and_stat
+from dulwich.objects import Blob
 
 
 class TreeTransformBase(TreeTransform):
@@ -59,19 +74,17 @@
         super(TreeTransformBase, self).__init__(tree, pb=pb)
         # mapping of trans_id => (sha1 of content, stat_value)
         self._observed_sha1s = {}
-        # Mapping of trans_id -> new file_id
-        self._new_id = {}
-        # Mapping of old file-id -> trans_id
-        self._non_present_ids = {}
-        # Mapping of new file_id -> trans_id
-        self._r_new_id = {}
+        # Set of versioned trans ids
+        self._versioned = set()
         # The trans_id that will be used as the tree root
-        if tree.is_versioned(''):
-            self._new_root = self.trans_id_tree_path('')
-        else:
-            self._new_root = None
+        self.root = self.trans_id_tree_path('')
         # Whether the target is case sensitive
         self._case_sensitive_target = case_sensitive
+        self._symlink_target = {}
+
+    @property
+    def mapping(self):
+        return self._tree.mapping
 
     def finalize(self):
         """Release the working tree lock, if held.
@@ -86,49 +99,16 @@
         self._tree.unlock()
         self._tree = None
 
-    def __get_root(self):
-        return self._new_root
-
-    root = property(__get_root)
-
     def create_path(self, name, parent):
         """Assign a transaction id to a new path"""
-        trans_id = self._assign_id()
+        trans_id = self.assign_id()
         unique_add(self._new_name, trans_id, name)
         unique_add(self._new_parent, trans_id, parent)
         return trans_id
 
     def adjust_root_path(self, name, parent):
         """Emulate moving the root by moving all children, instead.
-
-        We do this by undoing the association of root's transaction id with the
-        current tree.  This allows us to create a new directory with that
-        transaction id.  We unversion the root directory and version the
-        physically new directory, and hope someone versions the tree root
-        later.
         """
-        old_root = self._new_root
-        old_root_file_id = self.final_file_id(old_root)
-        # force moving all children of root
-        for child_id in self.iter_tree_children(old_root):
-            if child_id != parent:
-                self.adjust_path(self.final_name(child_id),
-                                 self.final_parent(child_id), child_id)
-            file_id = self.final_file_id(child_id)
-            if file_id is not None:
-                self.unversion_file(child_id)
-            self.version_file(child_id, file_id=file_id)
-
-        # the physical root needs a new transaction id
-        self._tree_path_ids.pop("")
-        self._tree_id_paths.pop(old_root)
-        self._new_root = self.trans_id_tree_path('')
-        if parent == old_root:
-            parent = self._new_root
-        self.adjust_path(name, parent, old_root)
-        self.create_directory(old_root)
-        self.version_file(old_root, file_id=old_root_file_id)
-        self.unversion_file(self._new_root)
 
     def fixup_new_roots(self):
         """Reinterpret requests to change the root directory
@@ -148,26 +128,12 @@
             return
         if len(new_roots) != 1:
             raise ValueError('A tree cannot have two roots!')
-        if self._new_root is None:
-            self._new_root = new_roots[0]
-            return
         old_new_root = new_roots[0]
         # unversion the new root's directory.
-        if self.final_kind(self._new_root) is None:
-            file_id = self.final_file_id(old_new_root)
-        else:
-            file_id = self.final_file_id(self._new_root)
-        if old_new_root in self._new_id:
+        if old_new_root in self._versioned:
             self.cancel_versioning(old_new_root)
         else:
             self.unversion_file(old_new_root)
-        # if, at this stage, root still has an old file_id, zap it so we can
-        # stick a new one in.
-        if (self.tree_file_id(self._new_root) is not None
-                and self._new_root not in self._removed_id):
-            self.unversion_file(self._new_root)
-        if file_id is not None:
-            self.version_file(self._new_root, file_id=file_id)
 
         # Now move children of new root into old root directory.
         # Ensure all children are registered with the transaction, but don't
@@ -175,7 +141,7 @@
         list(self.iter_tree_children(old_new_root))
         # Move all children of new root into old root directory.
         for child in self.by_parent().get(old_new_root, []):
-            self.adjust_path(self.final_name(child), self._new_root, child)
+            self.adjust_path(self.final_name(child), self.root, child)
 
         # Ensure old_new_root has no directory.
         if old_new_root in self._new_contents:
@@ -184,8 +150,8 @@
             self.delete_contents(old_new_root)
 
         # prevent deletion of root directory.
-        if self._new_root in self._removed_contents:
-            self.cancel_deletion(self._new_root)
+        if self.root in self._removed_contents:
+            self.cancel_deletion(self.root)
 
         # destroy path info for old_new_root.
         del self._new_parent[old_new_root]
@@ -199,24 +165,14 @@
         """
         if file_id is None:
             raise ValueError('None is not a valid file id')
-        if file_id in self._r_new_id and self._r_new_id[file_id] is not None:
-            return self._r_new_id[file_id]
-        else:
-            try:
-                path = self._tree.id2path(file_id)
-            except errors.NoSuchId:
-                if file_id in self._non_present_ids:
-                    return self._non_present_ids[file_id]
-                else:
-                    trans_id = self._assign_id()
-                    self._non_present_ids[file_id] = trans_id
-                    return trans_id
-            else:
-                return self.trans_id_tree_path(path)
+        path = self.mapping.parse_file_id(file_id)
+        return self.trans_id_tree_path(path)
 
     def version_file(self, trans_id, file_id=None):
         """Schedule a file to become versioned."""
-        raise NotImplementedError(self.version_file)
+        if trans_id in self._versioned:
+            raise errors.DuplicateKey(key=trans_id)
+        self._versioned.add(trans_id)
 
     def cancel_versioning(self, trans_id):
         """Undo a previous versioning of a file"""
@@ -233,53 +189,25 @@
             stale_ids = self._needs_rename.difference(self._new_name)
             stale_ids.difference_update(self._new_parent)
             stale_ids.difference_update(self._new_contents)
-            stale_ids.difference_update(self._new_id)
+            stale_ids.difference_update(self._versioned)
             needs_rename = self._needs_rename.difference(stale_ids)
             id_sets = (needs_rename, self._new_executability)
         else:
             id_sets = (self._new_name, self._new_parent, self._new_contents,
-                       self._new_id, self._new_executability)
+                       self._versioned, self._new_executability)
         for id_set in id_sets:
             new_ids.update(id_set)
         return sorted(FinalPaths(self).get_paths(new_ids))
 
-    def tree_file_id(self, trans_id):
-        """Determine the file id associated with the trans_id in the tree"""
-        path = self.tree_path(trans_id)
-        if path is None:
-            return None
-        # the file is old; the old id is still valid
-        if self._new_root == trans_id:
-            return self._tree.path2id('')
-        return self._tree.path2id(path)
-
     def final_is_versioned(self, trans_id):
-        return self.final_file_id(trans_id) is not None
-
-    def final_file_id(self, trans_id):
-        """Determine the file id after any changes are applied, or None.
-
-        None indicates that the file will not be versioned after changes are
-        applied.
-        """
-        try:
-            return self._new_id[trans_id]
-        except KeyError:
-            if trans_id in self._removed_id:
-                return None
-        return self.tree_file_id(trans_id)
-
-    def inactive_file_id(self, trans_id):
-        """Return the inactive file_id associated with a transaction id.
-        That is, the one in the tree or in non_present_ids.
-        The file_id may actually be active, too.
-        """
-        file_id = self.tree_file_id(trans_id)
-        if file_id is not None:
-            return file_id
-        for key, value in self._non_present_ids.items():
-            if value == trans_id:
-                return key
+        if trans_id in self._versioned:
+            return True
+        if trans_id in self._removed_id:
+            return False
+        orig_path = self.tree_path(trans_id)
+        if orig_path is None:
+            return False
+        return self._tree.is_versioned(orig_path)
 
     def find_raw_conflicts(self):
         """Find any violations of inventory or filesystem invariants"""
@@ -315,8 +243,11 @@
         for trans_id in self._removed_id:
             path = self.tree_path(trans_id)
             if path is not None:
-                if self._tree.stored_kind(path) == 'directory':
-                    parents.append(trans_id)
+                try:
+                    if self._tree.stored_kind(path) == 'directory':
+                        parents.append(trans_id)
+                except errors.NoSuchFile:
+                    pass
             elif self.tree_kind(trans_id) == 'directory':
                 parents.append(trans_id)
 
@@ -391,7 +322,7 @@
         However, existing entries with no contents are okay.
         """
         conflicts = []
-        for trans_id in self._new_id:
+        for trans_id in self._versioned:
             kind = self.final_kind(trans_id)
             if kind == 'symlink' and not self._tree.supports_symlinks():
                 # Ignore symlinks as they are not supported on this platform
@@ -597,7 +528,7 @@
     def _affected_ids(self):
         """Return the set of transform ids affected by the transform"""
         trans_ids = set(self._removed_id)
-        trans_ids.update(self._new_id)
+        trans_ids.update(self._versioned)
         trans_ids.update(self._removed_contents)
         trans_ids.update(self._new_contents)
         trans_ids.update(self._new_executability)
@@ -605,130 +536,78 @@
         trans_ids.update(self._new_parent)
         return trans_ids
 
-    def _get_file_id_maps(self):
-        """Return mapping of file_ids to trans_ids in the to and from states"""
-        trans_ids = self._affected_ids()
-        from_trans_ids = {}
-        to_trans_ids = {}
-        # Build up two dicts: trans_ids associated with file ids in the
-        # FROM state, vs the TO state.
-        for trans_id in trans_ids:
-            from_file_id = self.tree_file_id(trans_id)
-            if from_file_id is not None:
-                from_trans_ids[from_file_id] = trans_id
-            to_file_id = self.final_file_id(trans_id)
-            if to_file_id is not None:
-                to_trans_ids[to_file_id] = trans_id
-        return from_trans_ids, to_trans_ids
-
-    def _from_file_data(self, from_trans_id, from_versioned, from_path):
-        """Get data about a file in the from (tree) state
-
-        Return a (name, parent, kind, executable) tuple
-        """
-        from_path = self._tree_id_paths.get(from_trans_id)
-        if from_versioned:
-            # get data from working tree if versioned
-            from_entry = next(self._tree.iter_entries_by_dir(
-                specific_files=[from_path]))[1]
-            from_name = from_entry.name
-            from_parent = from_entry.parent_id
-        else:
-            from_entry = None
-            if from_path is None:
-                # File does not exist in FROM state
-                from_name = None
-                from_parent = None
-            else:
-                # File exists, but is not versioned.  Have to use path-
-                # splitting stuff
-                from_name = os.path.basename(from_path)
-                tree_parent = self.get_tree_parent(from_trans_id)
-                from_parent = self.tree_file_id(tree_parent)
-        if from_path is not None:
-            from_kind, from_executable, from_stats = \
-                self._tree._comparison_data(from_entry, from_path)
-        else:
-            from_kind = None
-            from_executable = False
-        return from_name, from_parent, from_kind, from_executable
-
-    def _to_file_data(self, to_trans_id, from_trans_id, from_executable):
-        """Get data about a file in the to (target) state
-
-        Return a (name, parent, kind, executable) tuple
-        """
-        to_name = self.final_name(to_trans_id)
-        to_kind = self.final_kind(to_trans_id)
-        to_parent = self.final_file_id(self.final_parent(to_trans_id))
-        if to_trans_id in self._new_executability:
-            to_executable = self._new_executability[to_trans_id]
-        elif to_trans_id == from_trans_id:
-            to_executable = from_executable
-        else:
-            to_executable = False
-        return to_name, to_parent, to_kind, to_executable
-
-    def iter_changes(self):
+    def iter_changes(self, want_unversioned=False):
         """Produce output in the same format as Tree.iter_changes.
 
         Will produce nonsensical results if invoked while inventory/filesystem
         conflicts (as reported by TreeTransform.find_raw_conflicts()) are present.
-
-        This reads the Transform, but only reproduces changes involving a
-        file_id.  Files that are not versioned in either of the FROM or TO
-        states are not reflected.
         """
         final_paths = FinalPaths(self)
-        from_trans_ids, to_trans_ids = self._get_file_id_maps()
+        trans_ids = self._affected_ids()
         results = []
-        # Now iterate through all active file_ids
-        for file_id in set(from_trans_ids).union(to_trans_ids):
+        # Now iterate through all active paths
+        for trans_id in trans_ids:
+            from_path = self.tree_path(trans_id)
             modified = False
-            from_trans_id = from_trans_ids.get(file_id)
             # find file ids, and determine versioning state
-            if from_trans_id is None:
+            if from_path is None:
                 from_versioned = False
-                from_trans_id = to_trans_ids[file_id]
             else:
-                from_versioned = True
-            to_trans_id = to_trans_ids.get(file_id)
-            if to_trans_id is None:
+                from_versioned = self._tree.is_versioned(from_path)
+            if not want_unversioned and not from_versioned:
+                from_path = None
+            to_path = final_paths.get_path(trans_id)
+            if to_path is None:
                 to_versioned = False
-                to_trans_id = from_trans_id
-            else:
-                to_versioned = True
-
-            if not from_versioned:
-                from_path = None
-            else:
-                from_path = self._tree_id_paths.get(from_trans_id)
-            if not to_versioned:
+            else:
+                to_versioned = self.final_is_versioned(trans_id)
+            if not want_unversioned and not to_versioned:
                 to_path = None
-            else:
-                to_path = final_paths.get_path(to_trans_id)
-
-            from_name, from_parent, from_kind, from_executable = \
-                self._from_file_data(from_trans_id, from_versioned, from_path)
-
-            to_name, to_parent, to_kind, to_executable = \
-                self._to_file_data(to_trans_id, from_trans_id, from_executable)
-
-            if from_kind != to_kind:
+
+            if from_versioned:
+                # get data from working tree if versioned
+                from_entry = next(self._tree.iter_entries_by_dir(
+                    specific_files=[from_path]))[1]
+                from_name = from_entry.name
+            else:
+                from_entry = None
+                if from_path is None:
+                    # File does not exist in FROM state
+                    from_name = None
+                else:
+                    # File exists, but is not versioned.  Have to use path-
+                    # splitting stuff
+                    from_name = os.path.basename(from_path)
+            if from_path is not None:
+                from_kind, from_executable, from_stats = \
+                    self._tree._comparison_data(from_entry, from_path)
+            else:
+                from_kind = None
+                from_executable = False
+
+            to_name = self.final_name(trans_id)
+            to_kind = self.final_kind(trans_id)
+            if trans_id in self._new_executability:
+                to_executable = self._new_executability[trans_id]
+            else:
+                to_executable = from_executable
+
+            if from_versioned and from_kind != to_kind:
                 modified = True
             elif to_kind in ('file', 'symlink') and (
-                    to_trans_id != from_trans_id
-                    or to_trans_id in self._new_contents):
+                    trans_id in self._new_contents):
                 modified = True
             if (not modified and from_versioned == to_versioned
-                and from_parent == to_parent and from_name == to_name
+                and from_path == to_path
+                and from_name == to_name
                     and from_executable == to_executable):
                 continue
+            if (from_path, to_path) == (None, None):
+                continue
             results.append(
-                InventoryTreeChange(
-                    file_id, (from_path, to_path), modified,
+                TreeChange(
+                    (from_path, to_path), modified,
                     (from_versioned, to_versioned),
-                    (from_parent, to_parent),
                     (from_name, to_name),
                     (from_kind, to_kind),
                     (from_executable, to_executable)))
@@ -743,7 +622,7 @@
         The tree is a snapshot, and altering the TreeTransform will invalidate
         it.
         """
-        raise NotImplementedError(self.get_preview_tree)
+        return GitPreviewTree(self)
 
     def commit(self, branch, message, merge_parents=None, strict=False,
                timestamp=None, timezone=None, committer=None, authors=None,
@@ -770,7 +649,7 @@
         """
         self._check_malformed()
         if strict:
-            unversioned = set(self._new_contents).difference(set(self._new_id))
+            unversioned = set(self._new_contents).difference(set(self._versioned))
             for trans_id in unversioned:
                 if not self.final_is_versioned(trans_id):
                     raise errors.StrictCommitFailed()
@@ -827,104 +706,6 @@
             return ()
         return (self._tree.get_file_lines(path),)
 
-    def serialize(self, serializer):
-        """Serialize this TreeTransform.
-
-        :param serializer: A Serialiser like pack.ContainerSerializer.
-        """
-        from .. import bencode
-        new_name = {k.encode('utf-8'): v.encode('utf-8')
-                    for k, v in self._new_name.items()}
-        new_parent = {k.encode('utf-8'): v.encode('utf-8')
-                      for k, v in self._new_parent.items()}
-        new_id = {k.encode('utf-8'): v
-                  for k, v in self._new_id.items()}
-        new_executability = {k.encode('utf-8'): int(v)
-                             for k, v in self._new_executability.items()}
-        tree_path_ids = {k.encode('utf-8'): v.encode('utf-8')
-                         for k, v in self._tree_path_ids.items()}
-        non_present_ids = {k: v.encode('utf-8')
-                           for k, v in self._non_present_ids.items()}
-        removed_contents = [trans_id.encode('utf-8')
-                            for trans_id in self._removed_contents]
-        removed_id = [trans_id.encode('utf-8')
-                      for trans_id in self._removed_id]
-        attribs = {
-            b'_id_number': self._id_number,
-            b'_new_name': new_name,
-            b'_new_parent': new_parent,
-            b'_new_executability': new_executability,
-            b'_new_id': new_id,
-            b'_tree_path_ids': tree_path_ids,
-            b'_removed_id': removed_id,
-            b'_removed_contents': removed_contents,
-            b'_non_present_ids': non_present_ids,
-            }
-        yield serializer.bytes_record(bencode.bencode(attribs),
-                                      ((b'attribs',),))
-        for trans_id, kind in sorted(self._new_contents.items()):
-            if kind == 'file':
-                with open(self._limbo_name(trans_id), 'rb') as cur_file:
-                    lines = cur_file.readlines()
-                parents = self._get_parents_lines(trans_id)
-                mpdiff = multiparent.MultiParent.from_lines(lines, parents)
-                content = b''.join(mpdiff.to_patch())
-            if kind == 'directory':
-                content = b''
-            if kind == 'symlink':
-                content = self._read_symlink_target(trans_id)
-                if not isinstance(content, bytes):
-                    content = content.encode('utf-8')
-            yield serializer.bytes_record(
-                content, ((trans_id.encode('utf-8'), kind.encode('ascii')),))
-
-    def deserialize(self, records):
-        """Deserialize a stored TreeTransform.
-
-        :param records: An iterable of (names, content) tuples, as per
-            pack.ContainerPushParser.
-        """
-        from .. import bencode
-        names, content = next(records)
-        attribs = bencode.bdecode(content)
-        self._id_number = attribs[b'_id_number']
-        self._new_name = {k.decode('utf-8'): v.decode('utf-8')
-                          for k, v in attribs[b'_new_name'].items()}
-        self._new_parent = {k.decode('utf-8'): v.decode('utf-8')
-                            for k, v in attribs[b'_new_parent'].items()}
-        self._new_executability = {
-            k.decode('utf-8'): bool(v)
-            for k, v in attribs[b'_new_executability'].items()}
-        self._new_id = {k.decode('utf-8'): v
-                        for k, v in attribs[b'_new_id'].items()}
-        self._r_new_id = {v: k for k, v in self._new_id.items()}
-        self._tree_path_ids = {}
-        self._tree_id_paths = {}
-        for bytepath, trans_id in attribs[b'_tree_path_ids'].items():
-            path = bytepath.decode('utf-8')
-            trans_id = trans_id.decode('utf-8')
-            self._tree_path_ids[path] = trans_id
-            self._tree_id_paths[trans_id] = path
-        self._removed_id = {trans_id.decode('utf-8')
-                            for trans_id in attribs[b'_removed_id']}
-        self._removed_contents = set(
-            trans_id.decode('utf-8')
-            for trans_id in attribs[b'_removed_contents'])
-        self._non_present_ids = {
-            k: v.decode('utf-8')
-            for k, v in attribs[b'_non_present_ids'].items()}
-        for ((trans_id, kind),), content in records:
-            trans_id = trans_id.decode('utf-8')
-            kind = kind.decode('ascii')
-            if kind == 'file':
-                mpdiff = multiparent.MultiParent.from_patch(content)
-                lines = mpdiff.to_lines(self._get_parents_texts(trans_id))
-                self.create_file(lines, trans_id)
-            if kind == 'directory':
-                self.create_directory(trans_id)
-            if kind == 'symlink':
-                self.create_symlink(content.decode('utf-8'), trans_id)
-
     def create_file(self, contents, trans_id, mode_id=None, sha1=None):
         """Schedule creation of a new file.
 
@@ -996,6 +777,16 @@
             elif c[0] == 'missing parent':
                 # TODO(jelmer): This should not make it to here
                 yield TextConflict(fp.get_path(c[2]))
+            elif c[0] == 'non-directory parent':
+                yield TextConflict(fp.get_path(c[2]))
+            elif c[0] == 'deleting parent':
+                # TODO(jelmer): This should not make it to here
+                yield TextConflict(fp.get_path(c[2]))
+            elif c[0] == 'parent loop':
+                # TODO(jelmer): This should not make it to here
+                yield TextConflict(fp.get_path(c[2]))
+            elif c[0] == 'path conflict':
+                yield TextConflict(fp.get_path(c[1]))
             else:
                 raise AssertionError('unknown conflict %s' % c[0])
 
@@ -1209,6 +1000,7 @@
                 path = None
             trace.warning(
                 'Unable to create symlink "%s" on this filesystem.' % (path,))
+            self._symlink_target[trans_id] = target
         # We add symlink to _new_contents even if they are unsupported
         # and not created. These entries are subsequently used to avoid
         # conflicts on platforms that don't support symlink
@@ -1233,6 +1025,88 @@
         handle_orphan = conf.get('transform.orphan_policy')
         handle_orphan(self, trans_id, parent_id)
 
+    def final_entry(self, trans_id):
+        is_versioned = self.final_is_versioned(trans_id)
+        fp = FinalPaths(self)
+        tree_path = fp.get_path(trans_id)
+        if trans_id in self._new_contents:
+            path = self._limbo_name(trans_id)
+            st = os.lstat(path)
+            kind = mode_kind(st.st_mode)
+            name = self.final_name(trans_id)
+            file_id = self._tree.mapping.generate_file_id(tree_path)
+            parent_id = self._tree.mapping.generate_file_id(os.path.dirname(tree_path))
+            if kind == 'directory':
+                return GitTreeDirectory(
+                    file_id, self.final_name(trans_id), parent_id=parent_id), is_versioned
+            executable = mode_is_executable(st.st_mode)
+            mode = object_mode(kind, executable)
+            blob = blob_from_path_and_stat(encode_git_path(path), st)
+            if kind == 'symlink':
+                return GitTreeSymlink(
+                    file_id, name, parent_id,
+                    decode_git_path(blob.data)), is_versioned
+            elif kind == 'file':
+                return GitTreeFile(
+                    file_id, name, executable=executable, parent_id=parent_id,
+                    git_sha1=blob.id, text_size=len(blob.data)), is_versioned
+            else:
+                raise AssertionError(kind)
+        elif trans_id in self._removed_contents:
+            return None, None
+        else:
+            orig_path = self.tree_path(trans_id)
+            if orig_path is None:
+                return None, None
+            file_id = self._tree.mapping.generate_file_id(tree_path)
+            if tree_path == '':
+                parent_id = None
+            else:
+                parent_id = self._tree.mapping.generate_file_id(os.path.dirname(tree_path))
+            try:
+                ie = next(self._tree.iter_entries_by_dir(
+                    specific_files=[orig_path]))[1]
+                ie.file_id = file_id
+                ie.parent_id = parent_id
+                return ie, is_versioned
+            except StopIteration:
+                try:
+                    if self.tree_kind(trans_id) == 'directory':
+                        return GitTreeDirectory(
+                            file_id, self.final_name(trans_id), parent_id=parent_id), is_versioned
+                except OSError as e:
+                    if e.errno != errno.ENOTDIR:
+                        raise
+                return None, None
+
+    def final_git_entry(self, trans_id):
+        if trans_id in self._new_contents:
+            path = self._limbo_name(trans_id)
+            st = os.lstat(path)
+            kind = mode_kind(st.st_mode)
+            if kind == 'directory':
+                return None, None
+            executable = mode_is_executable(st.st_mode)
+            mode = object_mode(kind, executable)
+            blob = blob_from_path_and_stat(encode_git_path(path), st)
+        elif trans_id in self._removed_contents:
+            return None, None
+        else:
+            orig_path = self.tree_path(trans_id)
+            kind = self._tree.kind(orig_path)
+            executable = self._tree.is_executable(orig_path)
+            mode = object_mode(kind, executable)
+            if kind == 'symlink':
+                contents = self._tree.get_symlink_target(orig_path)
+            elif kind == 'file':
+                contents = self._tree.get_file_text(orig_path)
+            elif kind == 'directory':
+                return None, None
+            else:
+                raise AssertionError(kind)
+            blob = Blob.from_string(contents)
+        return blob, mode
+
 
 class GitTreeTransform(DiskTreeTransform):
     """Represent a tree transformation.
@@ -1453,20 +1327,11 @@
         self._limbo_children_names[parent][filename] = trans_id
         return limbo_name
 
-    def version_file(self, trans_id, file_id=None):
-        """Schedule a file to become versioned."""
-        if file_id is None:
-            raise ValueError()
-        unique_add(self._new_id, trans_id, file_id)
-        unique_add(self._r_new_id, file_id, trans_id)
-
     def cancel_versioning(self, trans_id):
         """Undo a previous versioning of a file"""
-        file_id = self._new_id[trans_id]
-        del self._new_id[trans_id]
-        del self._r_new_id[file_id]
+        self._versioned.remove(trans_id)
 
-    def apply(self, no_conflicts=False, precomputed_delta=None, _mover=None):
+    def apply(self, no_conflicts=False, _mover=None):
         """Apply all changes to the inventory and filesystem.
 
         If filesystem or inventory conflicts are present, MalformedTransform
@@ -1476,8 +1341,6 @@
 
         :param no_conflicts: if True, the caller guarantees there are no
             conflicts, so no check is made.
-        :param precomputed_delta: An inventory delta to use instead of
-            calculating one.
         :param _mover: Supply an alternate FileMover, for testing
         """
         for hook in MutableTree.hooks['pre_transform']:
@@ -1486,14 +1349,9 @@
             self._check_malformed()
         self.rename_count = 0
         with ui.ui_factory.nested_progress_bar() as child_pb:
-            if precomputed_delta is None:
-                child_pb.update(gettext('Apply phase'), 0, 2)
-                changes = self._generate_transform_changes()
-                offset = 1
-            else:
-                changes = [
-                    (op, np, ie) for (op, np, fid, ie) in precomputed_delta]
-                offset = 0
+            child_pb.update(gettext('Apply phase'), 0, 2)
+            index_changes = self._generate_index_changes()
+            offset = 1
             if _mover is None:
                 mover = _FileMover()
             else:
@@ -1508,9 +1366,7 @@
                 raise
             else:
                 mover.apply_deletions()
-        if self.final_file_id(self.root) is None:
-            changes = [e for e in changes if e[0] != '']
-        self._tree._apply_transform_delta(changes)
+        self._tree._apply_index_changes(index_changes)
         self._done = True
         self.finalize()
         return _TransformResults(modified_paths, self.rename_count)
@@ -1593,97 +1449,354 @@
         self._new_contents.clear()
         return modified_paths
 
-    def _inventory_altered(self):
-        """Determine which trans_ids need new Inventory entries.
-
-        An new entry is needed when anything that would be reflected by an
-        inventory entry changes, including file name, file_id, parent file_id,
-        file kind, and the execute bit.
-
-        Some care is taken to return entries with real changes, not cases
-        where the value is deleted and then restored to its original value,
-        but some actually unchanged values may be returned.
-
-        :returns: A list of (path, trans_id) for all items requiring an
-            inventory change. Ordered by path.
-        """
+    def _generate_index_changes(self):
+        """Generate an inventory delta for the current transform."""
+        removed_id = set(self._removed_id)
+        removed_id.update(self._removed_contents)
+        changes = {}
         changed_ids = set()
-        # Find entries whose file_ids are new (or changed).
-        new_file_id = set(t for t in self._new_id
-                          if self._new_id[t] != self.tree_file_id(t))
-        for id_set in [self._new_name, self._new_parent, new_file_id,
+        for id_set in [self._new_name, self._new_parent,
                        self._new_executability]:
             changed_ids.update(id_set)
-        # removing implies a kind change
-        changed_kind = set(self._removed_contents)
+        for id_set in [self._new_name, self._new_parent]:
+            removed_id.update(id_set)
         # so does adding
-        changed_kind.intersection_update(self._new_contents)
+        changed_kind = set(self._new_contents)
         # Ignore entries that are already known to have changed.
         changed_kind.difference_update(changed_ids)
         #  to keep only the truly changed ones
         changed_kind = (t for t in changed_kind
                         if self.tree_kind(t) != self.final_kind(t))
-        # all kind changes will alter the inventory
         changed_ids.update(changed_kind)
-        # To find entries with changed parent_ids, find parents which existed,
-        # but changed file_id.
-        # Now add all their children to the set.
-        for parent_trans_id in new_file_id:
-            changed_ids.update(self.iter_tree_children(parent_trans_id))
-        return sorted(FinalPaths(self).get_paths(changed_ids))
-
-    def _generate_transform_changes(self):
-        """Generate an inventory delta for the current transform."""
-        changes = []
-        new_paths = self._inventory_altered()
-        total_entries = len(new_paths) + len(self._removed_id)
+        for t in changed_kind:
+            if self.final_kind(t) == 'directory':
+                removed_id.add(t)
+                changed_ids.remove(t)
+        new_paths = sorted(FinalPaths(self).get_paths(changed_ids))
+        total_entries = len(new_paths) + len(removed_id)
         with ui.ui_factory.nested_progress_bar() as child_pb:
-            for num, trans_id in enumerate(self._removed_id):
+            for num, trans_id in enumerate(removed_id):
                 if (num % 10) == 0:
                     child_pb.update(gettext('removing file'),
                                     num, total_entries)
-                if trans_id == self._new_root:
-                    file_id = self._tree.path2id('')
-                else:
-                    file_id = self.tree_file_id(trans_id)
-                # File-id isn't really being deleted, just moved
-                if file_id in self._r_new_id:
+                try:
+                    path = self._tree_id_paths[trans_id]
+                except KeyError:
                     continue
-                path = self._tree_id_paths[trans_id]
-                changes.append((path, None, None))
-            new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
-                                     new_paths)
+                changes[path] = (None, None, None, None)
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
                     child_pb.update(gettext('adding file'),
-                                    num + len(self._removed_id), total_entries)
-                file_id = new_path_file_ids[trans_id]
-                if file_id is None:
-                    continue
+                                    num + len(removed_id), total_entries)
+
                 kind = self.final_kind(trans_id)
                 if kind is None:
-                    kind = self._tree.stored_kind(self._tree.id2path(file_id))
-                parent_trans_id = self.final_parent(trans_id)
-                parent_file_id = new_path_file_ids.get(parent_trans_id)
-                if parent_file_id is None:
-                    parent_file_id = self.final_file_id(parent_trans_id)
-                if trans_id in self._new_reference_revision:
-                    new_entry = inventory.TreeReference(
-                        file_id,
-                        self._new_name[trans_id],
-                        self.final_file_id(self._new_parent[trans_id]),
-                        None, self._new_reference_revision[trans_id])
-                else:
-                    new_entry = inventory.make_entry(kind,
-                                                     self.final_name(trans_id),
-                                                     parent_file_id, file_id)
-                try:
-                    old_path = self._tree.id2path(new_entry.file_id)
-                except errors.NoSuchId:
-                    old_path = None
-                new_executability = self._new_executability.get(trans_id)
-                if new_executability is not None:
-                    new_entry.executable = new_executability
-                changes.append(
-                    (old_path, path, new_entry))
-        return changes
+                    continue
+                versioned = self.final_is_versioned(trans_id)
+                if not versioned:
+                    continue
+                executability = self._new_executability.get(trans_id)
+                reference_revision = self._new_reference_revision.get(trans_id)
+                symlink_target = self._symlink_target.get(trans_id)
+                changes[path] = (
+                    kind, executability, reference_revision, symlink_target)
+        return [(p, k, e, rr, st) for (p, (k, e, rr, st)) in changes.items()]
+
+
+class GitTransformPreview(GitTreeTransform):
+    """A TreeTransform for generating preview trees.
+
+    Unlike TreeTransform, this version works when the input tree is a
+    RevisionTree, rather than a WorkingTree.  As a result, it tends to ignore
+    unversioned files in the input tree.
+    """
+
+    def __init__(self, tree, pb=None, case_sensitive=True):
+        tree.lock_read()
+        limbodir = osutils.mkdtemp(prefix='bzr-limbo-')
+        DiskTreeTransform.__init__(self, tree, limbodir, pb, case_sensitive)
+
+    def canonical_path(self, path):
+        return path
+
+    def tree_kind(self, trans_id):
+        path = self.tree_path(trans_id)
+        if path is None:
+            return None
+        kind = self._tree.path_content_summary(path)[0]
+        if kind == 'missing':
+            kind = None
+        return kind
+
+    def _set_mode(self, trans_id, mode_id, typefunc):
+        """Set the mode of new file contents.
+        The mode_id is the existing file to get the mode from (often the same
+        as trans_id).  The operation is only performed if there's a mode match
+        according to typefunc.
+        """
+        # is it ok to ignore this?  probably
+        pass
+
+    def iter_tree_children(self, parent_id):
+        """Iterate through the entry's tree children, if any"""
+        try:
+            path = self._tree_id_paths[parent_id]
+        except KeyError:
+            return
+        try:
+            for child in self._tree.iter_child_entries(path):
+                childpath = joinpath(path, child.name)
+                yield self.trans_id_tree_path(childpath)
+        except errors.NoSuchFile:
+            return
+
+    def new_orphan(self, trans_id, parent_id):
+        raise NotImplementedError(self.new_orphan)
+
+
+class GitPreviewTree(PreviewTree, GitTree):
+    """Partial implementation of Tree to support show_diff_trees"""
+
+    def __init__(self, transform):
+        PreviewTree.__init__(self, transform)
+        self.store = transform._tree.store
+        self.mapping = transform._tree.mapping
+        self._final_paths = FinalPaths(transform)
+
+    def supports_setting_file_ids(self):
+        return False
+
+    def _supports_executable(self):
+        return self._transform._limbo_supports_executable()
+
+    def walkdirs(self, prefix=''):
+        pending = [self._transform.root]
+        while len(pending) > 0:
+            parent_id = pending.pop()
+            children = []
+            subdirs = []
+            prefix = prefix.rstrip('/')
+            parent_path = self._final_paths.get_path(parent_id)
+            for child_id in self._all_children(parent_id):
+                path_from_root = self._final_paths.get_path(child_id)
+                basename = self._transform.final_name(child_id)
+                kind = self._transform.final_kind(child_id)
+                if kind is not None:
+                    versioned_kind = kind
+                else:
+                    kind = 'unknown'
+                    versioned_kind = self._transform._tree.stored_kind(
+                        path_from_root)
+                if versioned_kind == 'directory':
+                    subdirs.append(child_id)
+                children.append((path_from_root, basename, kind, None,
+                                 versioned_kind))
+            children.sort()
+            if parent_path.startswith(prefix):
+                yield parent_path, children
+            pending.extend(sorted(subdirs, key=self._final_paths.get_path,
+                                  reverse=True))
+
+    def iter_changes(self, from_tree, include_unchanged=False,
+                     specific_files=None, pb=None, extra_trees=None,
+                     require_versioned=True, want_unversioned=False):
+        """See InterTree.iter_changes.
+
+        This has a fast path that is only used when the from_tree matches
+        the transform tree, and no fancy options are supplied.
+        """
+        return InterTree.get(from_tree, self).iter_changes(
+            include_unchanged=include_unchanged,
+            specific_files=specific_files,
+            pb=pb,
+            extra_trees=extra_trees,
+            require_versioned=require_versioned,
+            want_unversioned=want_unversioned)
+
+    def get_file(self, path):
+        """See Tree.get_file"""
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        if trans_id in self._transform._new_contents:
+            name = self._transform._limbo_name(trans_id)
+            return open(name, 'rb')
+        if trans_id in self._transform._removed_contents:
+            raise errors.NoSuchFile(path)
+        orig_path = self._transform.tree_path(trans_id)
+        return self._transform._tree.get_file(orig_path)
+
+    def get_symlink_target(self, path):
+        """See Tree.get_symlink_target"""
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        if trans_id not in self._transform._new_contents:
+            orig_path = self._transform.tree_path(trans_id)
+            return self._transform._tree.get_symlink_target(orig_path)
+        name = self._transform._limbo_name(trans_id)
+        return osutils.readlink(name)
+
+    def annotate_iter(self, path, default_revision=_mod_revision.CURRENT_REVISION):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            return None
+        orig_path = self._transform.tree_path(trans_id)
+        if orig_path is not None:
+            old_annotation = self._transform._tree.annotate_iter(
+                orig_path, default_revision=default_revision)
+        else:
+            old_annotation = []
+        try:
+            lines = self.get_file_lines(path)
+        except errors.NoSuchFile:
+            return None
+        return annotate.reannotate([old_annotation], lines, default_revision)
+
+    def get_file_text(self, path):
+        """Return the byte content of a file.
+
+        :param path: The path of the file.
+
+        :returns: A single byte string for the whole file.
+        """
+        with self.get_file(path) as my_file:
+            return my_file.read()
+
+    def get_file_lines(self, path):
+        """Return the content of a file, as lines.
+
+        :param path: The path of the file.
+        """
+        return osutils.split_lines(self.get_file_text(path))
+
+    def extras(self):
+        possible_extras = set(self._transform.trans_id_tree_path(p) for p
+                              in self._transform._tree.extras())
+        possible_extras.update(self._transform._new_contents)
+        possible_extras.update(self._transform._removed_id)
+        for trans_id in possible_extras:
+            if not self._transform.final_is_versioned(trans_id):
+                yield self._final_paths._determine_path(trans_id)
+
+    def path_content_summary(self, path):
+        trans_id = self._path2trans_id(path)
+        tt = self._transform
+        tree_path = tt.tree_path(trans_id)
+        kind = tt._new_contents.get(trans_id)
+        if kind is None:
+            if tree_path is None or trans_id in tt._removed_contents:
+                return 'missing', None, None, None
+            summary = tt._tree.path_content_summary(tree_path)
+            kind, size, executable, link_or_sha1 = summary
+        else:
+            link_or_sha1 = None
+            limbo_name = tt._limbo_name(trans_id)
+            if trans_id in tt._new_reference_revision:
+                kind = 'tree-reference'
+            if kind == 'file':
+                statval = os.lstat(limbo_name)
+                size = statval.st_size
+                if not tt._limbo_supports_executable():
+                    executable = False
+                else:
+                    executable = statval.st_mode & S_IEXEC
+            else:
+                size = None
+                executable = None
+            if kind == 'symlink':
+                link_or_sha1 = os.readlink(limbo_name)
+                if not isinstance(link_or_sha1, str):
+                    link_or_sha1 = link_or_sha1.decode(osutils._fs_enc)
+        executable = tt._new_executability.get(trans_id, executable)
+        return kind, size, executable, link_or_sha1
+
+    def get_file_mtime(self, path):
+        """See Tree.get_file_mtime"""
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        if trans_id not in self._transform._new_contents:
+            return self._transform._tree.get_file_mtime(
+                self._transform.tree_path(trans_id))
+        name = self._transform._limbo_name(trans_id)
+        statval = os.lstat(name)
+        return statval.st_mtime
+
+    def is_versioned(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            # It doesn't exist, so it's not versioned.
+            return False
+        if trans_id in self._transform._versioned:
+            return True
+        if trans_id in self._transform._removed_id:
+            return False
+        orig_path = self._transform.tree_path(trans_id)
+        return self._transform._tree.is_versioned(orig_path)
+
+    def iter_entries_by_dir(self, specific_files=None, recurse_nested=False):
+        if recurse_nested:
+            raise NotImplementedError(
+                'follow tree references not yet supported')
+
+        # This may not be a maximally efficient implementation, but it is
+        # reasonably straightforward.  An implementation that grafts the
+        # TreeTransform changes onto the tree's iter_entries_by_dir results
+        # might be more efficient, but requires tricky inferences about stack
+        # position.
+        for trans_id, path in self._list_files_by_dir():
+            entry, is_versioned = self._transform.final_entry(trans_id)
+            if entry is None:
+                continue
+            if not is_versioned and entry.kind != 'directory':
+                continue
+            if specific_files is not None and path not in specific_files:
+                continue
+            if entry is not None:
+                yield path, entry
+
+    def _list_files_by_dir(self):
+        todo = [ROOT_PARENT]
+        while len(todo) > 0:
+            parent = todo.pop()
+            children = list(self._all_children(parent))
+            paths = dict(zip(children, self._final_paths.get_paths(children)))
+            children.sort(key=paths.get)
+            todo.extend(reversed(children))
+            for trans_id in children:
+                yield trans_id, paths[trans_id][0]
+
+    def revision_tree(self, revision_id):
+        return self._transform._tree.revision_tree(revision_id)
+
+    def _stat_limbo_file(self, trans_id):
+        name = self._transform._limbo_name(trans_id)
+        return os.lstat(name)
+
+    def git_snapshot(self, want_unversioned=False):
+        extra = set()
+        os = []
+        for trans_id, path in self._list_files_by_dir():
+            if not self._transform.final_is_versioned(trans_id):
+                if not want_unversioned:
+                    continue
+                extra.add(path)
+            o, mode = self._transform.final_git_entry(trans_id)
+            if o is not None:
+                self.store.add_object(o)
+                os.append((encode_git_path(path), o.id, mode))
+        if not os:
+            return None, extra
+        return commit_tree(self.store, os), extra
+
+    def iter_child_entries(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id is None:
+            raise errors.NoSuchFile(path)
+        for child_trans_id in self._all_children(trans_id):
+            entry, is_versioned = self._transform.final_entry(trans_id)
+            if not is_versioned:
+                continue
+            if entry is not None:
+                yield entry

=== modified file 'breezy/git/tree.py'
--- breezy/git/tree.py	2020-08-10 15:00:17 +0000
+++ breezy/git/tree.py	2020-08-22 22:46:24 +0000
@@ -272,6 +272,39 @@
         """
         raise NotImplementedError(self.snapshot)
 
+    def preview_transform(self, pb=None):
+        from .transform import GitTransformPreview
+        return GitTransformPreview(self, pb=pb)
+
+    def find_related_paths_across_trees(self, paths, trees=[],
+                                        require_versioned=True):
+        if paths is None:
+            return None
+        if require_versioned:
+            trees = [self] + (trees if trees is not None else [])
+            unversioned = set()
+            for p in paths:
+                for t in trees:
+                    if t.is_versioned(p):
+                        break
+                else:
+                    unversioned.add(p)
+            if unversioned:
+                raise errors.PathsNotVersionedError(unversioned)
+        return filter(self.is_versioned, paths)
+
+    def _submodule_info(self):
+        if self._submodules is None:
+            try:
+                with self.get_file('.gitmodules') as f:
+                    config = GitConfigFile.from_file(f)
+                    self._submodules = {
+                        path: (url, section)
+                        for path, url, section in parse_submodules(config)}
+            except errors.NoSuchFile:
+                self._submodules = {}
+        return self._submodules
+
 
 class GitRevisionTree(revisiontree.RevisionTree, GitTree):
     """Revision tree implementation based on Git objects."""
@@ -298,18 +331,6 @@
     def git_snapshot(self, want_unversioned=False):
         return self.tree, set()
 
-    def _submodule_info(self):
-        if self._submodules is None:
-            try:
-                with self.get_file('.gitmodules') as f:
-                    config = GitConfigFile.from_file(f)
-                    self._submodules = {
-                        path: (url, section)
-                        for path, url, section in parse_submodules(config)}
-            except errors.NoSuchFile:
-                self._submodules = {}
-        return self._submodules
-
     def _get_submodule_repository(self, relpath):
         if not isinstance(relpath, bytes):
             raise TypeError(relpath)
@@ -440,18 +461,6 @@
         else:
             return True
 
-    def _submodule_info(self):
-        if self._submodules is None:
-            try:
-                with self.get_file('.gitmodules') as f:
-                    config = GitConfigFile.from_file(f)
-                    self._submodules = {
-                        path: (url, section)
-                        for path, url, section in parse_submodules(config)}
-            except errors.NoSuchFile:
-                self._submodules = {}
-        return self._submodules
-
     def list_files(self, include_root=False, from_dir=None, recursive=True,
                    recurse_nested=False):
         if self.tree is None:
@@ -667,23 +676,6 @@
         else:
             return (kind, None, None, None)
 
-    def find_related_paths_across_trees(self, paths, trees=[],
-                                        require_versioned=True):
-        if paths is None:
-            return None
-        if require_versioned:
-            trees = [self] + (trees if trees is not None else [])
-            unversioned = set()
-            for p in paths:
-                for t in trees:
-                    if t.is_versioned(p):
-                        break
-                else:
-                    unversioned.add(p)
-            if unversioned:
-                raise errors.PathsNotVersionedError(unversioned)
-        return filter(self.is_versioned, paths)
-
     def _iter_tree_contents(self, include_trees=False):
         if self.tree is None:
             return iter([])
@@ -735,10 +727,6 @@
                         mode_kind(mode)))
             yield path_decoded, children
 
-    def preview_transform(self, pb=None):
-        from .transform import GitTransformPreview
-        return GitTransformPreview(self, pb=pb)
-
 
 def tree_delta_from_git_changes(changes, mappings,
                                 specific_files=None,
@@ -973,8 +961,13 @@
             fileid = mapping.generate_file_id(newpath_decoded)
         else:
             fileid = None
+        if oldkind == 'directory' and newkind == 'directory':
+            modified = False
+        else:
+            modified = (oldsha != newsha) or (oldmode != newmode)
         yield InventoryTreeChange(
-            fileid, (oldpath_decoded, newpath_decoded), (oldsha != newsha),
+            fileid, (oldpath_decoded, newpath_decoded),
+            modified,
             (oldversioned, newversioned),
             (oldparent, newparent), (oldname, newname),
             (oldkind, newkind), (oldexe, newexe),
@@ -1193,18 +1186,6 @@
     def _read_submodule_head(self, path):
         raise NotImplementedError(self._read_submodule_head)
 
-    def _submodule_info(self):
-        if self._submodules is None:
-            try:
-                with self.get_file('.gitmodules') as f:
-                    config = GitConfigFile.from_file(f)
-                    self._submodules = {
-                        path: (url, section)
-                        for path, url, section in parse_submodules(config)}
-            except errors.NoSuchFile:
-                self._submodules = {}
-        return self._submodules
-
     def _lookup_index(self, encoded_path):
         if not isinstance(encoded_path, bytes):
             raise TypeError(encoded_path)
@@ -1240,11 +1221,32 @@
         # TODO(jelmer): Keep track of dirty per index
         self._index_dirty = True
 
-    def _index_add_entry(self, path, kind, flags=0, reference_revision=None):
+    def _apply_index_changes(self, changes):
+        for (path, kind, executability, reference_revision,
+             symlink_target) in changes:
+            if kind is None or kind == 'directory':
+                (index, subpath) = self._lookup_index(
+                    encode_git_path(path))
+                try:
+                    self._index_del_entry(index, subpath)
+                except KeyError:
+                    pass
+                else:
+                    self._versioned_dirs = None
+            else:
+                self._index_add_entry(
+                    path, kind,
+                    reference_revision=reference_revision,
+                    symlink_target=symlink_target)
+        self.flush()
+
+    def _index_add_entry(
+            self, path, kind, flags=0, reference_revision=None,
+            symlink_target=None):
         if kind == "directory":
             # Git indexes don't contain directories
             return
-        if kind == "file":
+        elif kind == "file":
             blob = Blob()
             try:
                 file, stat_val = self.get_file_with_stat(path)
@@ -1269,7 +1271,9 @@
                 # old index
                 stat_val = os.stat_result(
                     (stat.S_IFLNK, 0, 0, 0, 0, 0, 0, 0, 0, 0))
-            blob.set_raw_string(encode_git_path(self.get_symlink_target(path)))
+            if symlink_target is None:
+                symlink_target = self.get_symlink_target(path)
+            blob.set_raw_string(encode_git_path(symlink_target))
             # Add object to the repository if it didn't exist yet
             if blob.id not in self.store:
                 self.store.add_object(blob)
@@ -1561,25 +1565,6 @@
             self._versioned_dirs = None
             self.flush()
 
-    def find_related_paths_across_trees(self, paths, trees=[],
-                                        require_versioned=True):
-        if paths is None:
-            return None
-
-        if require_versioned:
-            trees = [self] + (trees if trees is not None else [])
-            unversioned = set()
-            for p in paths:
-                for t in trees:
-                    if t.is_versioned(p):
-                        break
-                else:
-                    unversioned.add(p)
-            if unversioned:
-                raise errors.PathsNotVersionedError(unversioned)
-
-        return filter(self.is_versioned, paths)
-
     def path_content_summary(self, path):
         """See Tree.path_content_summary."""
         try:
@@ -1606,17 +1591,21 @@
             return (kind, None, None, None)
 
     def stored_kind(self, relpath):
+        if relpath == '':
+            return 'directory'
         (index, index_path) = self._lookup_index(encode_git_path(relpath))
         if index is None:
-            return kind
+            return None
         try:
             mode = index[index_path].mode
         except KeyError:
-            return kind
+            for p in index:
+                if osutils.is_inside(
+                        decode_git_path(index_path), decode_git_path(p)):
+                    return 'directory'
+            return None
         else:
-            if S_ISGITLINK(mode):
-                return 'tree-reference'
-            return 'directory'
+            return mode_kind(mode)
 
     def kind(self, relpath):
         kind = osutils.file_kind(self.abspath(relpath))
@@ -1634,10 +1623,6 @@
         from .transform import GitTreeTransform
         return GitTreeTransform(self, pb=pb)
 
-    def preview_transform(self, pb=None):
-        from .transform import GitTransformPreview
-        return GitTransformPreview(self, pb=pb)
-
     def has_changes(self, _from_tree=None):
         """Quickly check that the tree contains at least one commitable change.
 

=== modified file 'breezy/git/workingtree.py'
--- breezy/git/workingtree.py	2020-08-10 15:00:17 +0000
+++ breezy/git/workingtree.py	2020-08-22 22:46:24 +0000
@@ -173,6 +173,12 @@
     def describe(self):
         return 'Text conflict in %(path)s' % self.__dict__
 
+    def __str__(self):
+        return self.describe()
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.path)
+
 
 class GitWorkingTree(MutableGitIndexTree, workingtree.WorkingTree):
     """A Git working tree."""
@@ -972,7 +978,6 @@
                 self._set_conflicted(path, path in by_path)
 
     def _set_conflicted(self, path, conflicted):
-        trace.mutter('change conflict: %r -> %r', path, conflicted)
         value = self.index[path]
         self._index_dirty = True
         if conflicted:
@@ -1156,26 +1161,6 @@
     def store_uncommitted(self):
         raise errors.StoringUncommittedNotSupported(self)
 
-    def _apply_transform_delta(self, changes):
-        for (old_path, new_path, ie) in changes:
-            if old_path is not None:
-                (index, old_subpath) = self._lookup_index(
-                    encode_git_path(old_path))
-                try:
-                    self._index_del_entry(index, old_subpath)
-                except KeyError:
-                    pass
-                else:
-                    self._versioned_dirs = None
-            if new_path is not None and ie.kind != 'directory':
-                if ie.kind == 'tree-reference':
-                    self._index_add_entry(
-                        new_path, ie.kind,
-                        reference_revision=ie.reference_revision)
-                else:
-                    self._index_add_entry(new_path, ie.kind)
-        self.flush()
-
     def annotate_iter(self, path,
                       default_revision=_mod_revision.CURRENT_REVISION):
         """See Tree.annotate_iter
@@ -1334,7 +1319,11 @@
     def get_reference_revision(self, path, branch=None):
         hexsha = self._read_submodule_head(path)
         if hexsha is None:
-            return _mod_revision.NULL_REVISION
+            (index, subpath) = self._lookup_index(
+                encode_git_path(path))
+            if subpath is None:
+                raise errors.NoSuchFile(path)
+            hexsha = index[subpath].sha
         return self.branch.lookup_foreign_revision_id(hexsha)
 
     def get_nested_tree(self, path):

=== modified file 'breezy/help_topics/en/configuration.txt'
--- breezy/help_topics/en/configuration.txt	2019-06-26 01:15:06 +0000
+++ breezy/help_topics/en/configuration.txt	2020-08-13 18:22:05 +0000
@@ -65,7 +65,7 @@
 ~~~~~~~
 
 Location of the Breezy log file. You can check the current location by
-running ``bzr version``.
+running ``brz version``.
 
 The log file contains debug information that is useful for diagnosing or
 reporting problems with Breezy.
@@ -87,15 +87,15 @@
 * the site specific plugin directory if applicable (containing
   the ``site`` plugins).
 
-If ``BZR_PLUGIN_PATH`` is set in any fashion, it will change the
+If ``BRZ_PLUGIN_PATH`` is set in any fashion, it will change the
 the way the plugin are searched. 
 
 As for the ``PATH`` variables, if multiple directories are
-specified in ``BZR_PLUGIN_PATH`` they should be separated by the
+specified in ``BRZ_PLUGIN_PATH`` they should be separated by the
 platform specific appropriate character (':' on Unix,
 ';' on windows)
 
-By default if ``BZR_PLUGIN_PATH`` is set, it replaces searching
+By default if ``BRZ_PLUGIN_PATH`` is set, it replaces searching
 in ``user``.  However it will continue to search in ``core`` and
 ``site`` unless they are explicitly removed.
 
@@ -118,19 +118,19 @@
 
 Overriding the default user plugin directory::
 
-  BZR_PLUGIN_PATH='/path/to/my/other/plugins'
+  BRZ_PLUGIN_PATH='/path/to/my/other/plugins'
 
 Disabling the site directory while retaining the user directory::
 
-  BZR_PLUGIN_PATH='-site:+user'
+  BRZ_PLUGIN_PATH='-site:+user'
 
 Disabling all plugins (better achieved with --no-plugins)::
 
-  BZR_PLUGIN_PATH='-user:-core:-site'
+  BRZ_PLUGIN_PATH='-user:-core:-site'
 
 Overriding the default site plugin directory::
 
-  BZR_PLUGIN_PATH='/path/to/my/site/plugins:-site':+user
+  BRZ_PLUGIN_PATH='/path/to/my/site/plugins:-site':+user
 
 BRZ_DISABLE_PLUGINS
 ~~~~~~~~~~~~~~~~~~~
@@ -138,15 +138,15 @@
 Under special circumstances (mostly when trying to diagnose a
 bug), it's better to disable a plugin (or several) rather than
 uninstalling them completely. Such plugins can be specified in
-the ``BZR_DISABLE_PLUGINS`` environment variable.
+the ``BRZ_DISABLE_PLUGINS`` environment variable.
 
-In that case, ``bzr`` will stop loading the specified plugins and
+In that case, ``brz`` will stop loading the specified plugins and
 will raise an import error if they are explicitly imported (by
 another plugin that depends on them for example).
 
 Disabling ``myplugin`` and ``yourplugin`` is achieved by::
 
-  BZR_DISABLE_PLUGINS='myplugin:yourplugin'
+  BRZ_DISABLE_PLUGINS='myplugin:yourplugin'
 
 BRZ_PLUGINS_AT
 ~~~~~~~~~~~~~~
@@ -156,11 +156,11 @@
 plugin. Since python requires that the directory containing the
 code is named like the plugin itself this make it impossible to
 use arbitrary directory names (using a two-level directory scheme
-is inconvenient). ``BZR_PLUGINS_AT`` allows such directories even
-if they don't appear in ``BZR_PLUGIN_PATH`` .
+is inconvenient). ``BRZ_PLUGINS_AT`` allows such directories even
+if they don't appear in ``BRZ_PLUGIN_PATH`` .
 
 Plugins specified in this environment variable takes precedence
-over the ones in ``BZR_PLUGIN_PATH``.
+over the ones in ``BRZ_PLUGIN_PATH``.
 
 The variable specified a list of ``plugin_name@plugin path``,
 ``plugin_name`` being the name of the plugin as it appears in
@@ -173,7 +173,7 @@
 ~~~~~~~~
 
 Using a specific version of ``myplugin``:
-``BZR_PLUGINS_AT='myplugin@/home/me/bugfixes/123456-myplugin``
+``BRZ_PLUGINS_AT='myplugin@/home/me/bugfixes/123456-myplugin``
 
 BRZPATH
 ~~~~~~~
@@ -199,7 +199,7 @@
 Configuration files are located in ``$HOME/.bazaar`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Breezy\2.0`` on
 Windows. (You can check the location for your system by using
-``bzr version``.)
+``brz version``.)
 
 There are three primary configuration files in this location:
 
@@ -315,7 +315,7 @@
 In both cases, when used in a directory like
 ``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::
 
-   $ bzr config mypush
+   $ brz config mypush
    lp:~vila/bzr/832013-expand-in-stack
 
 Another such option is ``basename`` which can be used like this::
@@ -326,7 +326,7 @@
 When used in a directory like
 ``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::
 
-   $ bzr config mypush
+   $ brz config mypush
    lp:~vila/bzr/832013-expand-in-stack
 
 Note that ``basename`` here refers to the base name of ``relpath`` which
@@ -342,7 +342,7 @@
 
 When used with a colocated branch named ``832013-expand-in-stack``, we'll get::
 
-  bzr config mypush
+  brz config mypush
   lp:~vila/bzr/832013-expand-in-stack
 
 When an option is local to a Section, it cannot be referred to from option
@@ -386,7 +386,7 @@
 
 ``authentication.conf`` allows one to specify credentials for
 remote servers. This can be used for all the supported transports and any part
-of bzr that requires authentication (smtp for example).
+of brz that requires authentication (smtp for example).
 
 The syntax of the file obeys the same rules as the others except for the
 option policies which don't apply.
@@ -422,9 +422,9 @@
 editor
 ~~~~~~
 
-The path of the editor that you wish to use if *bzr commit* is run without
+The path of the editor that you wish to use if *brz commit* is run without
 a commit message. This setting is trumped by the environment variable
-``BZR_EDITOR``, and overrides the ``VISUAL`` and ``EDITOR`` environment
+``BRZ_EDITOR``, and overrides the ``VISUAL`` and ``EDITOR`` environment
 variables.
 
 log_format
@@ -454,7 +454,7 @@
 create_signatures
 ~~~~~~~~~~~~~~~~~
 
-Defines the behaviour of signing revisions on commits.  By default bzr will not
+Defines the behaviour of signing revisions on commits.  By default brz will not
 sign new commits.
 
 always
@@ -514,7 +514,7 @@
 ~~~~~~~~~~~
 
 (Default: "localhost"). SMTP server to use when Breezy needs to send
-email, eg. with ``merge-directive --mail-to``, or the bzr-email plugin.
+email, eg. with ``merge-directive --mail-to``, or the email plugin.
 
 smtp_username, smtp_password
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -527,7 +527,7 @@
 locks.steal_dead
 ~~~~~~~~~~~~~~~~
 
-By default, bzr will automatically break locks held by processes from
+By default, brz will automatically break locks held by processes from
 the same machine and user that are no longer alive.  If disabled, it will
 print a message and you can break the lock manually, if you are satisfied
 the object is no longer in use.
@@ -536,7 +536,7 @@
 ~~~~~~~~~~~
 
 A mail client to use for sending merge requests.
-By default, bzr will try to use ``mapi`` on Windows.  On other platforms, it
+By default, brz will try to use ``mapi`` on Windows.  On other platforms, it
 will try ``xdg-email``. If either of these fails, it will fall back to
 ``editor``.
 
@@ -553,7 +553,7 @@
 
 :default: See above.
 :editor: Use your editor to compose the merge request.  This also uses
-    your commit id, (see ``bzr whoami``), smtp_server and (optionally)
+    your commit id, (see ``brz whoami``), smtp_server and (optionally)
     smtp_username and smtp_password.
 :mapi: Use your preferred e-mail client on Windows.
 :xdg-email: Use xdg-email to run your preferred mail program
@@ -569,20 +569,20 @@
 ~~~~~~~~~~~~~
 
 The branch you intend to submit your current work to.  This is automatically
-set by ``bzr send``, and is also used by the ``submit:`` revision spec.  This
+set by ``brz send``, and is also used by the ``submit:`` revision spec.  This
 should usually be set on a per-branch or per-location basis.
 
 public_branch
 ~~~~~~~~~~~~~
 
 A publically-accessible version of this branch (implying that this version is
-not publically-accessible).  Used (and set) by ``bzr send``.
+not publically-accessible).  Used (and set) by ``brz send``.
 
 suppress_warnings
 ~~~~~~~~~~~~~~~~~
 
 A list of strings, each string represent a warning that can be emitted by
-bzr. Mentioning a warning in this list tells bzr to not emit it.
+brz. Mentioning a warning in this list tells bzr to not emit it.
 
 Valid values:
 
@@ -597,7 +597,7 @@
 default_format
 ~~~~~~~~~~~~~~
 
-A format name for the default format used when creating branches.  See ``bzr
+A format name for the default format used when creating branches.  See ``brz
 help formats`` for possible values.
 
 
@@ -626,9 +626,9 @@
 If set to "True" then revisions can only be appended to the log, not
 removed.  A branch with this setting enabled can only pull from another
 branch if the other branch's log is a longer version of its own.  This is
-normally set by ``bzr init --append-revisions-only``. If you set it
+normally set by ``brz init --append-revisions-only``. If you set it
 manually, use either 'True' or 'False' (case-sensitive) to maintain
-compatibility with previous bzr versions (older than 2.2).
+compatibility with previous brz versions (older than 2.2).
 
 parent_location
 ~~~~~~~~~~~~~~~
@@ -705,13 +705,13 @@
 following problem::
 
 
-  $ bzr config bzr.mergetool.kdiff3='kdiff3 {base} {this} {other} -o {result}'
-  $ bzr config bzr.mergetool.kdiff3
-  bzr: ERROR: Option base is not defined while expanding "kdiff3 {base} {this} {other} -o {result}".
+  $ brz config bzr.mergetool.kdiff3='kdiff3 {base} {this} {other} -o {result}'
+  $ brz config bzr.mergetool.kdiff3
+  brz: ERROR: Option base is not defined while expanding "kdiff3 {base} {this} {other} -o {result}".
 
 To avoid this, ``config`` can be instructed not to try expanding variables::
 
-  $ bzr config --all bzr.mergetool.kdiff3
+  $ brz config --all bzr.mergetool.kdiff3
   branch:
     bzr.mergetool.kdiff3 = kdiff3 {base} {this} {other} -o {result}
 
@@ -720,7 +720,7 @@
 ~~~~~~~~~~~~~~~~~~~~~
 
 Specifies which external merge tool (as defined above) should be selected by
-default in tools such as ``bzr qconflicts``.
+default in tools such as ``brz qconflicts``.
 
 For example::
 

=== modified file 'breezy/help_topics/en/conflict-types.txt'
--- breezy/help_topics/en/conflict-types.txt	2017-05-21 14:47:52 +0000
+++ breezy/help_topics/en/conflict-types.txt	2020-08-13 18:22:05 +0000
@@ -374,4 +374,4 @@
 It is possible (though very rare) for Bazaar to raise a MalformedTransform
 exception.  This means that Bazaar encountered a filesystem conflict that it was
 unable to resolve.  This usually indicates a bug.  Please let us know if you
-encounter this.  Our bug tracker is at https://launchpad.net/bzr/+bugs
+encounter this.  Our bug tracker is at https://launchpad.net/brz/+bugs

=== modified file 'breezy/help_topics/en/glossary.txt'
--- breezy/help_topics/en/glossary.txt	2018-11-19 22:05:30 +0000
+++ breezy/help_topics/en/glossary.txt	2020-08-13 18:22:05 +0000
@@ -91,8 +91,8 @@
     changes. For these cases, a user may instead prefer a cherrypick, which
     gives the user the option to apply only those revisions that are wanted.
 
-    As well as altering the working tree, bzr's merge command will also get all
-    of the revisions that a branch has, and add metadata indicating that a
+    As well as altering the working tree, Breezy's merge command will also get
+    all of the revisions that a branch has, and add metadata indicating that a
     merge has been performed.
 
   merge, criss-cross
@@ -120,13 +120,13 @@
   revision
     A revision is one state of the code.
 
-    bzr is snapshot based, meaning that conceptually it stores the whole state
-    of the tree at each revision. This is in contrast to changeset based, where
-    a revision refers to just the changes that were made in that revision. I
-    said "conceptually", as internally bzr uses delta compression to reduce the
-    amount of data that it stores, and so may only record the changes in a
-    revision. However you must always think of a revision as referring to a
-    whole snapshot.
+    Breezy is snapshot based, meaning that conceptually it stores the whole
+    state of the tree at each revision. This is in contrast to changeset based,
+    where a revision refers to just the changes that were made in that
+    revision. I said "conceptually", as internally Breezy uses delta
+    compression to reduce the amount of data that it stores, and so may only
+    record the changes in a revision. However you must always think of a
+    revision as referring to a whole snapshot.
 
     A revision also contains some metadata. In particular it stores the
     committer, and the timestamp when the commit was made, the log message that
@@ -136,7 +136,7 @@
     number of revision properties, these is extra information that can be
     stored with the revision and is immutable, some uses of them are to store
     the author of the patch if it wasn't the committer (when given using the
-    --author option of bzr commit), or to store information about the bugs
+    --author option of brz commit), or to store information about the bugs
     fixed in that revision if using the bug-tracker integration.
 
     A revision also has a name, referred to as a revision id. This is a

=== modified file 'breezy/merge.py'
--- breezy/merge.py	2020-08-10 15:00:17 +0000
+++ breezy/merge.py	2020-08-22 22:46:24 +0000
@@ -790,7 +790,15 @@
         self.active_hooks = [hook for hook in hooks if hook is not None]
         with ui.ui_factory.nested_progress_bar() as child_pb:
             for num, (file_id, changed, paths3, parents3, names3,
-                      executable3) in enumerate(entries):
+                      executable3, copied) in enumerate(entries):
+                if copied:
+                    # Treat copies as simple adds for now
+                    paths3 = (None, paths3[1], None)
+                    parents3 = (None, parents3[1], None)
+                    names3 = (None, names3[1], None)
+                    executable3 = (None, executable3[1], None)
+                    changed = True
+                    copied = False
                 trans_id = self.tt.trans_id_file_id(file_id)
                 # Try merging each entry
                 child_pb.update(gettext('Preparing file merge'),
@@ -861,7 +869,7 @@
             executable3 = change.executable + (this_executable,)
             yield (
                 (change.file_id, change.changed_content, paths3,
-                 parents3, names3, executable3))
+                 parents3, names3, executable3, change.copied))
 
     def _entries_lca(self):
         """Gather data about files modified between multiple trees.
@@ -871,7 +879,7 @@
 
         For the multi-valued entries, the format will be (BASE, [lca1, lca2])
 
-        :return: [(file_id, changed, paths, parents, names, executable)], where:
+        :return: [(file_id, changed, paths, parents, names, executable, copied)], where:
 
             * file_id: Simple file_id of the entry
             * changed: Boolean, True if the kind or contents changed else False
@@ -1041,7 +1049,10 @@
                            ((base_ie.name, lca_names),
                             other_ie.name, this_ie.name),
                            ((base_ie.executable, lca_executable),
-                            other_ie.executable, this_ie.executable)
+                            other_ie.executable, this_ie.executable),
+                           # Copy detection is not yet supported, so nothing is
+                           # a copy:
+                           False
                            )
 
     def write_modified(self, results):
@@ -1465,7 +1476,6 @@
         if self.this_tree.supports_content_filtering():
             filter_tree_path = this_path
         else:
-            # Skip the id2path lookup for older formats
             filter_tree_path = None
 
         file_group = []

=== modified file 'breezy/plugins/bash_completion/README.txt'
--- breezy/plugins/bash_completion/README.txt	2017-06-02 21:40:29 +0000
+++ breezy/plugins/bash_completion/README.txt	2020-08-13 18:22:05 +0000
@@ -2,14 +2,14 @@
 
   Copyright (C) 2010 Canonical Ltd
 
-  This file is part of bzr-bash-completion
+  This file is part of Breezy.
 
-  bzr-bash-completion free software: you can redistribute it and/or
+  Breezy is free software: you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation, either version 2 of the
   License, or (at your option) any later version.
 
-  bzr-bash-completion is distributed in the hope that it will be
+  Breezy is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
@@ -18,14 +18,14 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ==========================
-bzr bash-completion plugin
+bash-completion plugin
 ==========================
 
 This plugin generates a shell function which can be used by bash to
 automatically complete the currently typed command when the user
 presses the completion key (usually tab).
 
-It is intended as a bzr plugin, but can be used to some extend as a
+It is intended as a Breezy plugin, but can be used to some extend as a
 standalone python script as well.
 
 | Copyright (C) 2009, 2010 Canonical Ltd
@@ -36,22 +36,10 @@
 Bundled and standalone versions
 -------------------------------
 
-This plugin has been merged_ into the main source tree of Bazaar.
-Starting with the bzr 2.3 series, a common bzr installation will
+This plugin has been merged_ into the main source tree of Breezy.
+Starting with the brz 2.3 series, a common brz installation will
 include this plugin.
 
-There is still a standalone version available. It makes the plugin
-available for users of older bzr versions. When using both versions,
-local configuration might determine which version actually gets used,
-and some installations might even overwrite one another, so don't use
-the standalone version if you have the bundled one, unless you know
-what you are doing. Some effort will be made to keep the two versions
-reasonably in sync for some time yet.
-
-This text here documents the bundled version.
-
-.. _merged: http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/revision/5240
-
 -----
 Using
 -----
@@ -60,11 +48,11 @@
 -----------------
 
 This is the preferred method of generating the completion function, as
-it will ensure proper bzr initialization.
+it will ensure proper brz initialization.
 
 ::
 
-  eval "`bzr bash-completion`"
+  eval "`brz bash-completion`"
 
 Lazy initialization
 -------------------
@@ -74,7 +62,7 @@
 To avoid this problem, you can delay the generation of the completion
 function until you actually need it.
 
-To do so, source the file ``contrib/bash/bzr`` shipped with the bzr
+To do so, source the file ``contrib/bash/brz`` shipped with the brz
 source distribution from your ``~/.bashrc`` file
 or add it to your ``~/.bash_completion`` if
 your setup uses such a file. On a system-wide installation, the
@@ -82,7 +70,7 @@
 completion scripts.
 
 Note that the full completion function is generated only once per
-shell session. If you update your bzr installation or change the set
+shell session. If you update your brz installation or change the set
 of installed plugins, then you might wish to regenerate the completion
 function manually as described above in order for completion to take
 these changes into account.
@@ -94,10 +82,10 @@
 The plugin is designed to generate a completion function
 containing all the required information about the possible
 completions. This is usually only done once when bash
-initializes. After that, no more invocations of bzr are required. This
+initializes. After that, no more invocations of brz are required. This
 makes the function much faster than a possible implementation talking
-to bzr for each and every completion. On the other hand, this has the
-effect that updates to bzr or its plugins won't show up in the
+to brz for each and every completion. On the other hand, this has the
+effect that updates to brz or its plugins won't show up in the
 completions immediately, but only after the completion function has
 been regenerated.
 
@@ -106,7 +94,7 @@
 -------
 
 As this is built upon a bash completion script originally included in
-the bzr source tree, and as the bzr sources are covered by the GPL 2,
+the brz source tree, and as the brz sources are covered by the GPL 2,
 this plugin here is licensed under these same terms.
 
 If you require a more liberal license, you'll have to contact all
@@ -119,9 +107,9 @@
 
 The plugin was created by Martin von Gagern in 2009, building on a
 static completion function of very limited scope distributed together
-with bzr.
+with brz.
 
-A version of it was merged into the bzr source tree in May 2010.
+A version of it was merged into the brz source tree in May 2010.
 
 ----------
 References

=== modified file 'breezy/plugins/fastimport/doc/notes.txt'
--- breezy/plugins/fastimport/doc/notes.txt	2008-03-18 04:44:00 +0000
+++ breezy/plugins/fastimport/doc/notes.txt	2020-08-13 18:22:05 +0000
@@ -1,5 +1,5 @@
 =======================
-Notes on bzr-fastimport
+Notes on brz-fastimport
 =======================
 
 ..contents::

=== modified file 'breezy/tests/per_tree/test_transform.py'
--- breezy/tests/per_tree/test_transform.py	2020-08-06 22:38:03 +0000
+++ breezy/tests/per_tree/test_transform.py	2020-08-15 22:46:49 +0000
@@ -30,8 +30,10 @@
     )
 from ...tree import (
     find_previous_path,
+    TreeChange,
     )
 
+from breezy.bzr.inventorytree import InventoryTreeChange
 
 from breezy.tests.per_tree import TestCaseWithTree
 
@@ -46,11 +48,6 @@
 
 class TestTransformPreview(TestCaseWithTree):
 
-    def setUp(self):
-        super(TestTransformPreview, self).setUp()
-        if not self.workingtree_format.supports_setting_file_ids:
-            self.skipTest('test not compatible with non-file-id trees yet')
-
     def create_tree(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('a', b'content 1')])
@@ -153,20 +150,39 @@
                            (False, False), False)],
                          list(preview_tree.iter_changes(revision_tree)))
 
+    def assertTreeChanges(self, expected, actual, tree):
+        # TODO(jelmer): Turn this into a matcher?
+        actual = list(actual)
+        if tree.supports_setting_file_ids():
+            self.assertEqual(expected, actual)
+        else:
+            expected = [
+                TreeChange(path=c.path, changed_content=c.changed_content,
+                           versioned=c.versioned, name=c.name,
+                           kind=c.kind, executable=c.executable,
+                           copied=c.copied) for c in expected]
+            actual = [
+                TreeChange(path=c.path, changed_content=c.changed_content,
+                           versioned=c.versioned, name=c.name,
+                           kind=c.kind, executable=c.executable,
+                           copied=c.copied) for c in actual]
+            self.assertEqual(expected, actual)
+
     def test_include_unchanged_succeeds(self):
         revision_tree, preview_tree = self.get_tree_and_preview_tree()
         changes = preview_tree.iter_changes(revision_tree,
                                             include_unchanged=True)
 
         root_id = revision_tree.path2id('')
-        root_entry = (root_id, ('', ''), False, (True, True), (None, None),
-                      ('', ''), ('directory', 'directory'), (False, False), False)
-        a_entry = (revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
-                   (root_id, root_id), ('a', 'a'), ('file', 'file'),
-                   (False, False), False)
+        root_entry = InventoryTreeChange(
+            root_id, ('', ''), False, (True, True), (None, None),
+            ('', ''), ('directory', 'directory'), (False, False), False)
+        a_entry = InventoryTreeChange(
+            revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+            (root_id, root_id), ('a', 'a'), ('file', 'file'),
+            (False, False), False)
 
- 
-        self.assertEqual([root_entry, a_entry], list(changes))
+        self.assertTreeChanges([root_entry, a_entry], changes, preview_tree)
 
     def test_specific_files(self):
         revision_tree, preview_tree = self.get_tree_and_preview_tree()
@@ -177,7 +193,6 @@
                    (root_id, root_id), ('a', 'a'), ('file', 'file'),
                    (False, False), False)
 
- 
         self.assertEqual([a_entry], list(changes))
 
     def test_want_unversioned(self):
@@ -185,9 +200,10 @@
         changes = preview_tree.iter_changes(revision_tree,
                                             want_unversioned=True)
         root_id = revision_tree.path2id('')
-        a_entry = (revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
-                   (root_id, root_id), ('a', 'a'), ('file', 'file'),
-                   (False, False), False)
+        a_entry = InventoryTreeChange(
+            revision_tree.path2id('a'), ('a', 'a'), True, (True, True),
+            (root_id, root_id), ('a', 'a'), ('file', 'file'),
+            (False, False), False)
 
         self.assertEqual([a_entry], list(changes))
 
@@ -304,7 +320,9 @@
         preview.new_file('new', preview.trans_id_tree_path('unchanged'),
                          [b'contents'], b'new-id')
         preview_tree = preview.get_preview_tree()
-        self.assertEqual(b'new-id', preview_tree.path2id('unchanged/new'))
+        self.assertTrue(preview_tree.is_versioned('unchanged/new'))
+        if self.workingtree_format.supports_setting_file_ids:
+            self.assertEqual(b'new-id', preview_tree.path2id('unchanged/new'))
 
     def test_path2id_moved(self):
         tree = self.make_branch_and_tree('tree')
@@ -339,7 +357,7 @@
         self.assertEqual(
             'new_name/child',
             find_previous_path(tree, preview_tree, 'old_name/child'))
-        if tree.supports_setting_file_ids:
+        if tree.supports_setting_file_ids():
             self.assertEqual(
                 tree.path2id('old_name/child'),
                 preview_tree.path2id('new_name/child'))
@@ -516,8 +534,11 @@
             (revid1, b'a\n'),
         ]
         annotation = preview_tree.annotate_iter(
+            'newname', default_revision=b'me:')
+        self.assertEqual(expected, annotation)
+        annotation = preview_tree.annotate_iter(
             'file', default_revision=b'me:')
-        self.assertEqual(expected, annotation)
+        self.assertIs(None, annotation)
 
     def test_annotate_deleted(self):
         tree = self.make_branch_and_tree('tree')
@@ -542,8 +563,8 @@
 
     def test_is_executable(self):
         preview = self.get_empty_preview()
-        preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
-        preview.set_executability(True, preview.trans_id_file_id(b'file-id'))
+        trans_id = preview.new_file('file', preview.root, [b'a\nb\nc\n'], b'file-id')
+        preview.set_executability(True, trans_id)
         preview_tree = preview.get_preview_tree()
         self.assertEqual(True, preview_tree.is_executable('file'))
 
@@ -568,6 +589,8 @@
         preview.create_file([b'b\nc\nd\ne\n'], trans_id)
         self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
         tree_a = preview.get_preview_tree()
+        if not getattr(tree_a, 'plan_file_merge', None):
+            self.skipTest('tree does not support file merge planning')
         tree_a.set_parent_ids([base_id])
         self.addCleanup(tree_b.lock_read().unlock)
         self.assertEqual([
@@ -592,6 +615,8 @@
         preview.create_file([b'b\nc\nd\ne\n'], trans_id)
         self.build_tree_contents([('wtb/file', b'a\nc\nd\nf\n')])
         tree_a = preview.get_preview_tree()
+        if not getattr(tree_a, 'plan_file_merge', None):
+            self.skipTest('tree does not support file merge planning')
         tree_a.set_parent_ids([base_id])
         self.addCleanup(tree_b.lock_read().unlock)
         self.assertEqual([
@@ -624,6 +649,7 @@
         preview.new_file('new-versioned-file', preview.root, [b'contents'],
                          b'new-versioned-id')
         tree = preview.get_preview_tree()
+        self.assertEquals({'existing-file'}, set(work_tree.extras()))
         preview.unversion_file(preview.trans_id_tree_path('removed-file'))
         self.assertEqual({'new-file', 'removed-file', 'existing-file'},
                          set(tree.extras()))

=== modified file 'breezy/tests/per_workingtree/test_commit.py'
--- breezy/tests/per_workingtree/test_commit.py	2020-08-10 03:38:37 +0000
+++ breezy/tests/per_workingtree/test_commit.py	2020-08-15 22:46:49 +0000
@@ -169,7 +169,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         changes = list(tree.iter_changes(tree.basis_tree()))
-        self.assertEqual(1, len(changes))
+        self.assertEqual(1, len(changes), changes)
         self.assertEqual((None, 'a/b'), changes[0].path)
 
     def test_commit_sets_last_revision(self):

=== modified file 'breezy/tests/per_workingtree/test_merge_from_branch.py'
--- breezy/tests/per_workingtree/test_merge_from_branch.py	2020-08-10 03:38:37 +0000
+++ breezy/tests/per_workingtree/test_merge_from_branch.py	2020-08-15 22:46:49 +0000
@@ -101,7 +101,7 @@
         tree_a.lock_read()
         self.addCleanup(tree_a.unlock)
         changes = list(tree_a.iter_changes(tree_a.basis_tree()))
-        self.assertEqual(1, len(changes))
+        self.assertEqual(1, len(changes), changes)
 
     def test_merge_type(self):
         this = self.make_branch_and_tree('this')

=== modified file 'breezy/tests/per_workingtree/test_smart_add.py'
--- breezy/tests/per_workingtree/test_smart_add.py	2020-02-07 02:14:30 +0000
+++ breezy/tests/per_workingtree/test_smart_add.py	2020-08-22 22:46:24 +0000
@@ -322,6 +322,9 @@
         self.build_tree_contents([('t1/file', b'content in t1')])
         t1.commit('Changing file in t1')
         t1.merge_from_branch(tb.branch)
+        fnames = ['file.%s' % s for s in ('BASE', 'THIS', 'OTHER')]
+        for fn in fnames:
+            self.assertPathExists(os.path.join(t1.basedir, fn))
         return t1
 
     def test_cant_add_generated_files_implicitly(self):

=== modified file 'breezy/tests/per_workingtree/test_transform.py'
--- breezy/tests/per_workingtree/test_transform.py	2020-08-10 15:00:17 +0000
+++ breezy/tests/per_workingtree/test_transform.py	2020-08-22 22:46:24 +0000
@@ -28,6 +28,7 @@
     transform,
     urlutils,
     )
+from ...tree import TreeChange
 from ...bzr.conflicts import (
     DeletingParent,
     DuplicateEntry,
@@ -69,6 +70,8 @@
     TransformRenameFailed,
 )
 
+from breezy.bzr.inventorytree import InventoryTreeChange
+
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -77,8 +80,6 @@
 
     def setUp(self):
         super(TestTreeTransform, self).setUp()
-        if not self.workingtree_format.supports_setting_file_ids:
-            self.skipTest('test not compatible with non-file-id trees yet')
         self.wt = self.make_branch_and_tree('wt')
 
     def transform(self):
@@ -88,6 +89,9 @@
 
     def transform_for_sha1_test(self):
         trans, root = self.transform()
+        if getattr(self.wt, '_observed_sha1', None) is None:
+            raise tests.TestNotApplicable(
+                'wt format does not use _observed_sha1')
         self.wt.lock_tree_write()
         self.addCleanup(self.wt.unlock)
         contents = [b'just some content\n']
@@ -130,16 +134,21 @@
         self.assertEqual(imaginary_id, imaginary_id2)
         self.assertEqual(root, transform.get_tree_parent(imaginary_id))
         self.assertEqual('directory', transform.final_kind(root))
-        self.assertEqual(self.wt.path2id(''), transform.final_file_id(root))
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id(''), transform.final_file_id(root))
         trans_id = transform.create_path('name', root)
-        self.assertIs(transform.final_file_id(trans_id), None)
+        if self.wt.supports_setting_file_ids():
+            self.assertIs(transform.final_file_id(trans_id), None)
+        self.assertFalse(transform.final_is_versioned(trans_id))
         self.assertIs(None, transform.final_kind(trans_id))
         transform.create_file([b'contents'], trans_id)
         transform.set_executability(True, trans_id)
         transform.version_file(trans_id, file_id=b'my_pretties')
         self.assertRaises(DuplicateKey, transform.version_file,
                           trans_id, file_id=b'my_pretties')
-        self.assertEqual(transform.final_file_id(trans_id), b'my_pretties')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(transform.final_file_id(trans_id), b'my_pretties')
+        self.assertTrue(transform.final_is_versioned(trans_id))
         self.assertEqual(transform.final_parent(trans_id), root)
         self.assertIs(transform.final_parent(root), ROOT_PARENT)
         self.assertIs(transform.get_tree_parent(root), ROOT_PARENT)
@@ -153,20 +162,25 @@
         modified_paths = transform.apply().modified_paths
         with self.wt.get_file('name') as f:
             self.assertEqual(b'contents', f.read())
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id('name'), b'my_pretties')
         self.assertIs(self.wt.is_executable('name'), True)
         self.assertIs(self.wt.is_executable('name2'), False)
         self.assertEqual('directory', file_kind(self.wt.abspath('oz')))
         self.assertEqual(len(modified_paths), 3)
-        tree_mod_paths = [self.wt.abspath(self.wt.id2path(f)) for f in
-                          (b'ozzie', b'my_pretties', b'my_pretties2')]
-        self.assertSubset(tree_mod_paths, modified_paths)
+        if self.wt.supports_setting_file_ids():
+            tree_mod_paths = [self.wt.abspath(self.wt.id2path(f)) for f in
+                              (b'ozzie', b'my_pretties', b'my_pretties2')]
+            self.assertSubset(tree_mod_paths, modified_paths)
         # is it safe to finalize repeatedly?
         transform.finalize()
         transform.finalize()
 
     def test_apply_informs_tree_of_observed_sha1(self):
         trans, root, contents, sha1 = self.transform_for_sha1_test()
+        from ...bzr.workingtree import InventoryWorkingTree
+        if not isinstance(self.wt, InventoryWorkingTree):
+            self.skipTest('not a bzr working tree')
         trans_id = trans.new_file('file1', root, contents, file_id=b'file1-id',
                                   sha1=sha1)
         calls = []
@@ -300,7 +314,10 @@
         with tt:
             tt.new_directory('', ROOT_PARENT, b'new-root-id')
             tt.fixup_new_roots()
-            self.assertNotEqual(b'new-root-id', tt.final_file_id(tt.root))
+            if self.wt.has_versioned_directories():
+                self.assertTrue(tt.final_is_versioned(tt.root))
+            if self.wt.supports_setting_file_ids():
+                self.assertNotEqual(b'new-root-id', tt.final_file_id(tt.root))
 
     def test_retain_existing_root_added_file(self):
         tt, root = self.transform()
@@ -314,7 +331,7 @@
         transform.new_directory('', ROOT_PARENT, None)
         transform.delete_contents(transform.root)
         transform.fixup_new_roots()
-        self.assertNotIn(transform.root, transform._new_id)
+        self.assertNotIn(transform.root, getattr(transform, '_new_id', []))
 
     def test_remove_root_fixup(self):
         transform, root = self.transform()
@@ -337,7 +354,8 @@
         transform.unversion_file(root)
         transform.fixup_new_roots()
         self.assertIs(None, transform.final_kind(root))
-        self.assertIs(None, transform.final_file_id(root))
+        if self.wt.supports_setting_file_ids():
+            self.assertIs(None, transform.final_file_id(root))
 
     def test_apply_retains_root_directory(self):
         # Do not attempt to delete the physical root directory, because that
@@ -352,6 +370,8 @@
 
     def test_apply_retains_file_id(self):
         transform, root = self.transform()
+        if not self.wt.supports_setting_file_ids():
+            self.skipTest('format does not support file ids')
         old_root_id = transform.tree_file_id(root)
         transform.unversion_file(root)
         transform.apply()
@@ -387,14 +407,19 @@
         self.assertRaises(ReusingTransform, transform.find_raw_conflicts)
         with open(self.wt.abspath('name'), 'r') as f:
             self.assertEqual('contents', f.read())
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
         self.assertIs(self.wt.is_executable('name'), True)
-        self.assertEqual(self.wt.path2id('oz'), b'oz-id')
-        self.assertEqual(self.wt.path2id('oz/dorothy'), b'dorothy-id')
-        self.assertEqual(self.wt.path2id('oz/dorothy/toto'), b'toto-id')
+        self.assertTrue(self.wt.is_versioned('name'))
+        self.assertTrue(self.wt.is_versioned('oz'))
+        self.assertTrue(self.wt.is_versioned('oz/dorothy'))
+        self.assertTrue(self.wt.is_versioned('oz/dorothy/toto'))
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+            self.assertEqual(self.wt.path2id('oz'), b'oz-id')
+            self.assertEqual(self.wt.path2id('oz/dorothy'), b'dorothy-id')
+            self.assertEqual(self.wt.path2id('oz/dorothy/toto'), b'toto-id')
 
-        self.assertEqual(b'toto-contents',
-                         self.wt.get_file('oz/dorothy/toto').read())
+        with self.wt.get_file('oz/dorothy/toto') as f:
+            self.assertEqual(b'toto-contents', f.read())
         self.assertIs(self.wt.is_executable('oz/dorothy/toto'), False)
 
     def test_tree_reference(self):
@@ -403,15 +428,17 @@
         if not tree.supports_tree_reference():
             raise tests.TestNotApplicable(
                 'Tree format does not support references')
+        nested_tree = self.make_branch_and_tree('nested')
+        nested_revid = nested_tree.commit('commit')
 
         trans_id = transform.new_directory('reference', root, b'subtree-id')
-        transform.set_tree_reference(b'subtree-revision', trans_id)
+        transform.set_tree_reference(nested_revid, trans_id)
         transform.apply()
         tree.lock_read()
         self.addCleanup(tree.unlock)
         self.assertEqual(
-            b'subtree-revision',
-            tree.root_inventory.get_entry(b'subtree-id').reference_revision)
+            nested_revid,
+            tree.get_reference_revision('reference'))
 
     def test_conflicts(self):
         transform, root = self.transform()
@@ -427,17 +454,29 @@
                          [('non-directory parent', trans_id)])
         tinman_id = transform.trans_id_tree_path('tinman')
         transform.adjust_path('name', tinman_id, trans_id2)
-        self.assertEqual(transform.find_raw_conflicts(),
-                         [('unversioned parent', tinman_id),
-                          ('missing parent', tinman_id)])
+        if self.wt.has_versioned_directories():
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('unversioned parent', tinman_id),
+                              ('missing parent', tinman_id)])
+        else:
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('missing parent', tinman_id)])
         lion_id = transform.create_path('lion', root)
-        self.assertEqual(transform.find_raw_conflicts(),
-                         [('unversioned parent', tinman_id),
-                          ('missing parent', tinman_id)])
+        if self.wt.has_versioned_directories():
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('unversioned parent', tinman_id),
+                              ('missing parent', tinman_id)])
+        else:
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('missing parent', tinman_id)])
         transform.adjust_path('name', lion_id, trans_id2)
-        self.assertEqual(transform.find_raw_conflicts(),
-                         [('unversioned parent', lion_id),
-                          ('missing parent', lion_id)])
+        if self.wt.has_versioned_directories():
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('unversioned parent', lion_id),
+                              ('missing parent', lion_id)])
+        else:
+            self.assertEqual(transform.find_raw_conflicts(),
+                             [('missing parent', lion_id)])
         transform.version_file(lion_id, file_id=b"Courage")
         self.assertEqual(transform.find_raw_conflicts(),
                          [('missing parent', lion_id),
@@ -461,7 +500,8 @@
         transform.set_executability(None, oz_id)
         tip_id = transform.new_file('tip', oz_id, [b'ozma'], b'tip-id')
         transform.apply()
-        self.assertEqual(self.wt.path2id('name'), b'my_pretties')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id('name'), b'my_pretties')
         with open(self.wt.abspath('name'), 'rb') as f:
             self.assertEqual(b'contents', f.read())
         transform2, root = self.transform()
@@ -471,11 +511,14 @@
         fp = FinalPaths(transform2)
         self.assertTrue('oz/tip' in transform2._tree_path_ids)
         self.assertEqual(fp.get_path(newtip), pathjoin('oz', 'tip'))
-        self.assertEqual(len(result), 2)
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(len(result), 2)
+            self.assertEqual((result[1][0], result[1][2]),
+                             ('duplicate id', newtip))
+        else:
+            self.assertEqual(len(result), 1)
         self.assertEqual((result[0][0], result[0][1]),
                          ('duplicate', newtip))
-        self.assertEqual((result[1][0], result[1][2]),
-                         ('duplicate id', newtip))
         transform2.finalize()
         transform3 = self.wt.transform()
         self.addCleanup(transform3.finalize)
@@ -626,8 +669,11 @@
         self.addCleanup(unversion.finalize)
         parent = unversion.trans_id_tree_path('parent')
         unversion.unversion_file(parent)
-        self.assertEqual(unversion.find_raw_conflicts(),
-                         [('unversioned parent', parent_id)])
+        if self.wt.has_versioned_directories():
+            self.assertEqual(unversion.find_raw_conflicts(),
+                             [('unversioned parent', parent_id)])
+        else:
+            self.assertEqual(unversion.find_raw_conflicts(), [])
         file_id = unversion.trans_id_tree_path('parent/child')
         unversion.unversion_file(file_id)
         unversion.apply()
@@ -666,9 +712,9 @@
         mfile2 = mangle_tree.trans_id_tree_path('moving_file2')
         mangle_tree.adjust_path('mfile2', newdir, mfile2)
         mangle_tree.new_file('newfile', newdir, [b'hello3'], b'dfile')
-        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
         self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
-        self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
         mangle_tree.apply()
         with open(self.wt.abspath('name1'), 'r') as f:
             self.assertEqual(f.read(), 'hello2')
@@ -678,13 +724,15 @@
         self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
         with open(mfile2_path, 'r') as f:
             self.assertEqual(f.read(), 'later2')
-        self.assertEqual(self.wt.id2path(b'mfile2'), 'new_directory/mfile2')
-        self.assertEqual(self.wt.path2id('new_directory/mfile2'), b'mfile2')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.id2path(b'mfile2'), 'new_directory/mfile2')
+            self.assertEqual(self.wt.path2id('new_directory/mfile2'), b'mfile2')
         newfile_path = self.wt.abspath(pathjoin('new_directory', 'newfile'))
         with open(newfile_path, 'r') as f:
             self.assertEqual(f.read(), 'hello3')
-        self.assertEqual(self.wt.path2id('dying_directory'), b'ddir')
-        self.assertIs(self.wt.path2id('dying_directory/dying_file'), None)
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id('dying_directory'), b'ddir')
+            self.assertIs(self.wt.path2id('dying_directory/dying_file'), None)
         mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
 
     def test_both_rename(self):
@@ -767,7 +815,10 @@
         name2 = replace.new_file('name2', root, [b'hello2'], b'name1')
         conflicts = replace.find_raw_conflicts()
         name1 = replace.trans_id_tree_path('name1')
-        self.assertEqual(conflicts, [('duplicate id', name1, name2)])
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(conflicts, [('duplicate id', name1, name2)])
+        else:
+            self.assertEqual(conflicts, [])
         resolve_conflicts(replace)
         replace.apply()
 
@@ -789,7 +840,8 @@
                          [('non-file executability', wiz_id)])
         transform.set_executability(None, wiz_id)
         transform.apply()
-        self.assertEqual(self.wt.path2id(ozpath(link_name1)), b'wizard-id')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(self.wt.path2id(ozpath(link_name1)), b'wizard-id')
         self.assertEqual('symlink',
                          file_kind(self.wt.abspath(ozpath(link_name1))))
         self.assertEqual(link_target2,
@@ -840,87 +892,104 @@
         conflicts.delete_versioned(oz)
         emerald = conflicts.trans_id_tree_path('oz/emeraldcity')
         # set up MissingParent conflict
-        munchkincity = conflicts.trans_id_file_id(b'munchkincity-id')
+        if conflicts._tree.supports_setting_file_ids():
+            munchkincity = conflicts.trans_id_file_id(b'munchkincity-id')
+        else:
+            munchkincity = conflicts.assign_id()
         conflicts.adjust_path('munchkincity', root, munchkincity)
         conflicts.new_directory('auntem', munchkincity, b'auntem-id')
         # set up parent loop
         conflicts.adjust_path('emeraldcity', emerald, emerald)
-        return conflicts, emerald, oz, old_dorothy, new_dorothy
+        return conflicts, emerald, oz, old_dorothy, new_dorothy, munchkincity
 
     def test_conflict_resolution(self):
-        conflicts, emerald, oz, old_dorothy, new_dorothy =\
+        conflicts, emerald, oz, old_dorothy, new_dorothy, munchkincity =\
             self.get_conflicted()
         resolve_conflicts(conflicts)
         self.assertEqual(conflicts.final_name(old_dorothy), 'dorothy.moved')
-        self.assertIs(conflicts.final_file_id(old_dorothy), None)
+        if self.wt.supports_setting_file_ids():
+            self.assertIs(conflicts.final_file_id(old_dorothy), None)
+            self.assertEqual(conflicts.final_file_id(new_dorothy), b'dorothy-id')
         self.assertEqual(conflicts.final_name(new_dorothy), 'dorothy')
-        self.assertEqual(conflicts.final_file_id(new_dorothy), b'dorothy-id')
         self.assertEqual(conflicts.final_parent(emerald), oz)
         conflicts.apply()
 
     def test_cook_conflicts(self):
-        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        tt, emerald, oz, old_dorothy, new_dorothy, munchkincity = self.get_conflicted()
         raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
-        duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved',
-                                   'dorothy', None, b'dorothy-id')
-        self.assertEqual(cooked_conflicts[0], duplicate)
-        duplicate_id = DuplicateID('Unversioned existing file',
-                                   'dorothy.moved', 'dorothy', None,
-                                   b'dorothy-id')
-        self.assertEqual(cooked_conflicts[1], duplicate_id)
-        missing_parent = MissingParent('Created directory', 'munchkincity',
-                                       b'munchkincity-id')
-        deleted_parent = DeletingParent('Not deleting', 'oz', b'oz-id')
-        self.assertEqual(cooked_conflicts[2], missing_parent)
-        unversioned_parent = UnversionedParent('Versioned directory',
-                                               'munchkincity',
-                                               b'munchkincity-id')
-        unversioned_parent2 = UnversionedParent('Versioned directory', 'oz',
-                                                b'oz-id')
-        self.assertEqual(cooked_conflicts[3], unversioned_parent)
-        parent_loop = ParentLoop(
-            'Cancelled move', 'oz/emeraldcity',
-            'oz/emeraldcity', b'emerald-id', b'emerald-id')
-        self.assertEqual(cooked_conflicts[4], deleted_parent)
-        self.assertEqual(cooked_conflicts[5], unversioned_parent2)
-        self.assertEqual(cooked_conflicts[6], parent_loop)
-        self.assertEqual(len(cooked_conflicts), 7)
+        cooked_conflicts = list(tt.cook_conflicts(raw_conflicts))
+        if self.wt.supports_setting_file_ids():
+            duplicate = DuplicateEntry('Moved existing file to', 'dorothy.moved',
+                                       'dorothy', None, b'dorothy-id')
+            self.assertEqual(cooked_conflicts[0], duplicate)
+            duplicate_id = DuplicateID('Unversioned existing file',
+                                       'dorothy.moved', 'dorothy', None,
+                                       b'dorothy-id')
+            self.assertEqual(cooked_conflicts[1], duplicate_id)
+            missing_parent = MissingParent('Created directory', 'munchkincity',
+                                           b'munchkincity-id')
+            deleted_parent = DeletingParent('Not deleting', 'oz', b'oz-id')
+            self.assertEqual(cooked_conflicts[2], missing_parent)
+            unversioned_parent = UnversionedParent('Versioned directory',
+                                                   'munchkincity',
+                                                   b'munchkincity-id')
+            unversioned_parent2 = UnversionedParent('Versioned directory', 'oz',
+                                                    b'oz-id')
+            self.assertEqual(cooked_conflicts[3], unversioned_parent)
+            parent_loop = ParentLoop(
+                'Cancelled move', 'oz/emeraldcity',
+                'oz/emeraldcity', b'emerald-id', b'emerald-id')
+            self.assertEqual(cooked_conflicts[4], deleted_parent)
+            self.assertEqual(cooked_conflicts[5], unversioned_parent2)
+            self.assertEqual(cooked_conflicts[6], parent_loop)
+            self.assertEqual(len(cooked_conflicts), 7)
+        else:
+            self.assertEqual(
+                set([c.path for c in cooked_conflicts]),
+                set(['oz/emeraldcity', 'oz', 'munchkincity', 'dorothy.moved']))
         tt.finalize()
 
     def test_string_conflicts(self):
-        tt, emerald, oz, old_dorothy, new_dorothy = self.get_conflicted()
+        tt, emerald, oz, old_dorothy, new_dorothy, munchkincity = self.get_conflicted()
         raw_conflicts = resolve_conflicts(tt)
-        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
+        cooked_conflicts = list(tt.cook_conflicts(raw_conflicts))
         tt.finalize()
         conflicts_s = [str(c) for c in cooked_conflicts]
         self.assertEqual(len(cooked_conflicts), len(conflicts_s))
-        self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
-                                         'Moved existing file to '
-                                         'dorothy.moved.')
-        self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
-                                         'Unversioned existing file '
-                                         'dorothy.moved.')
-        self.assertEqual(conflicts_s[2], 'Conflict adding files to'
-                                         ' munchkincity.  Created directory.')
-        self.assertEqual(conflicts_s[3], 'Conflict because munchkincity is not'
-                                         ' versioned, but has versioned'
-                                         ' children.  Versioned directory.')
-        self.assertEqualDiff(
-            conflicts_s[4], "Conflict: can't delete oz because it"
-                            " is not empty.  Not deleting.")
-        self.assertEqual(conflicts_s[5], 'Conflict because oz is not'
-                                         ' versioned, but has versioned'
-                                         ' children.  Versioned directory.')
-        self.assertEqual(conflicts_s[6], 'Conflict moving oz/emeraldcity into'
-                                         ' oz/emeraldcity. Cancelled move.')
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
+                                             'Moved existing file to '
+                                             'dorothy.moved.')
+            self.assertEqual(conflicts_s[1], 'Conflict adding id to dorothy.  '
+                                             'Unversioned existing file '
+                                             'dorothy.moved.')
+            self.assertEqual(conflicts_s[2], 'Conflict adding files to'
+                                             ' munchkincity.  Created directory.')
+            self.assertEqual(conflicts_s[3], 'Conflict because munchkincity is not'
+                                             ' versioned, but has versioned'
+                                             ' children.  Versioned directory.')
+            self.assertEqualDiff(
+                conflicts_s[4], "Conflict: can't delete oz because it"
+                                " is not empty.  Not deleting.")
+            self.assertEqual(conflicts_s[5], 'Conflict because oz is not'
+                                             ' versioned, but has versioned'
+                                             ' children.  Versioned directory.')
+            self.assertEqual(conflicts_s[6], 'Conflict moving oz/emeraldcity into'
+                                             ' oz/emeraldcity. Cancelled move.')
+        else:
+            self.assertEqual(
+                {'Text conflict in dorothy.moved',
+                 'Text conflict in munchkincity',
+                 'Text conflict in oz',
+                 'Text conflict in oz/emeraldcity'},
+                set([c for c in conflicts_s]))
 
     def prepare_wrong_parent_kind(self):
         tt, root = self.transform()
         tt.new_file('parent', root, [b'contents'], b'parent-id')
         tt.apply()
         tt, root = self.transform()
-        parent_id = tt.trans_id_file_id(b'parent-id')
+        parent_id = tt.trans_id_tree_path('parent')
         tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
         return tt
 
@@ -933,12 +1002,19 @@
         raw_conflicts = resolve_conflicts(tt)
         self.assertEqual({('non-directory parent', 'Created directory',
                            'new-3')}, raw_conflicts)
-        cooked_conflicts = tt.cook_conflicts(raw_conflicts)
-        self.assertEqual([NonDirectoryParent('Created directory', 'parent.new',
-                                             b'parent-id')], cooked_conflicts)
+        cooked_conflicts = list(tt.cook_conflicts(raw_conflicts))
+        from ...bzr.workingtree import InventoryWorkingTree
+        if isinstance(tt._tree, InventoryWorkingTree):
+            self.assertEqual([NonDirectoryParent('Created directory', 'parent.new',
+                                                 b'parent-id')], cooked_conflicts)
+        else:
+            self.assertEqual(1, len(cooked_conflicts))
+            self.assertEqual('parent.new', cooked_conflicts[0].path)
         tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
+        if self.wt.has_versioned_directories():
+            self.assertFalse(self.wt.is_versioned('parent'))
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
 
     def test_resolve_conflicts_wrong_new_parent_kind(self):
         tt, root = self.transform()
@@ -946,15 +1022,18 @@
         tt.new_file('child,', parent_id, [b'contents2'], b'file-id')
         tt.apply()
         tt, root = self.transform()
-        parent_id = tt.trans_id_file_id(b'parent-id')
+        parent_id = tt.trans_id_tree_path('parent')
         tt.delete_contents(parent_id)
         tt.create_file([b'contents'], parent_id)
         raw_conflicts = resolve_conflicts(tt)
         self.assertEqual({('non-directory parent', 'Created directory',
                            'new-3')}, raw_conflicts)
         tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
+        if self.wt.has_versioned_directories():
+            self.assertFalse(self.wt.is_versioned('parent'))
+            self.assertTrue(self.wt.is_versioned('parent.new'))
+        if self.wt.supports_setting_file_ids():
+            self.assertEqual(b'parent-id', self.wt.path2id('parent.new'))
 
     def test_resolve_conflicts_wrong_parent_kind_unversioned(self):
         tt, root = self.transform()
@@ -967,14 +1046,15 @@
         tt.create_file([b'contents'], parent_id)
         resolve_conflicts(tt)
         tt.apply()
-        self.assertFalse(self.wt.is_versioned('parent'))
-        self.assertFalse(self.wt.is_versioned('parent.new'))
+        if self.wt.has_versioned_directories():
+            self.assertFalse(self.wt.is_versioned('parent'))
+            self.assertFalse(self.wt.is_versioned('parent.new'))
 
     def test_resolve_conflicts_missing_parent(self):
         wt = self.make_branch_and_tree('.')
         tt = wt.transform()
         self.addCleanup(tt.finalize)
-        parent = tt.trans_id_file_id(b'parent-id')
+        parent = tt.assign_id()
         tt.new_file('file', parent, [b'Contents'])
         raw_conflicts = resolve_conflicts(tt)
         # Since the directory doesn't exist it's seen as 'missing'.  So
@@ -1017,9 +1097,9 @@
                                  b'myfile-id')
         create.apply()
         rename, root = self.transform()
-        old = rename.trans_id_file_id(b'old-id')
+        old = rename.trans_id_tree_path('old-parent')
         rename.adjust_path('new', root, old)
-        myfile = rename.trans_id_file_id(b'myfile-id')
+        myfile = rename.trans_id_tree_path('old-parent/intermediate/myfile')
         rename.set_executability(True, myfile)
         rename.apply()
 
@@ -1040,8 +1120,8 @@
             self.skipTest("Can't force a permissions error on rename")
         # now transform to rename
         rename_transform, root_id = self.transform()
-        file_trans_id = rename_transform.trans_id_file_id(b'myfile-id')
-        dir_id = rename_transform.trans_id_file_id(b'first-id')
+        file_trans_id = rename_transform.trans_id_tree_path('myfile')
+        dir_id = rename_transform.trans_id_tree_path('first-dir')
         rename_transform.adjust_path('newname', dir_id, file_trans_id)
         e = self.assertRaises(TransformRenameFailed,
                               rename_transform.apply)
@@ -1142,20 +1222,35 @@
         transform.apply()
         transform, root = self.transform()
         try:
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             old = transform.trans_id_tree_path('old')
             transform.unversion_file(old)
-            self.assertEqual([(b'id-1', ('old', None), False, (True, False),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (True, True), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform, [
+                    InventoryTreeChange(
+                        b'id-1', ('old', None), False, (True, False),
+                        (root_id, root_id), ('old', 'old'), ('file', 'file'),
+                        (True, True), False)])
             transform.new_directory('new', root, b'id-1')
-            self.assertEqual([(b'id-1', ('old', 'new'), True, (True, True),
-                               (root_id, root_id), ('old', 'new'),
-                               ('file', 'directory'),
-                               (True, False), False)],
-                             list(transform.iter_changes()))
+            if transform._tree.supports_setting_file_ids():
+                self.assertTreeChanges(
+                    transform,
+                    [InventoryTreeChange(
+                        b'id-1', ('old', 'new'), True, (True, True),
+                        (root_id, root_id), ('old', 'new'),
+                        ('file', 'directory'),
+                        (True, False), False)])
+            else:
+                self.assertTreeChanges(
+                    transform,
+                    [TreeChange(
+                        (None, 'new'), False, (False, True),
+                        (None, 'new'), (None, 'directory'),
+                        (False, False), False),
+                     TreeChange(
+                        ('old', None), False, (True, False),
+                        ('old', 'old'), ('file', 'file'),
+                        (True, True), False)])
         finally:
             transform.finalize()
 
@@ -1168,11 +1263,15 @@
         try:
             old = transform.trans_id_tree_path('old')
             transform.version_file(old, file_id=b'id-1')
-            self.assertEqual([(b'id-1', (None, 'old'), False, (False, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            changes = list(transform.iter_changes())
+            self.assertEqual(1, len(changes))
+            self.assertEqual((None, 'old'), changes[0].path)
+            self.assertEqual(False, changes[0].changed_content)
+            self.assertEqual((False, True), changes[0].versioned)
+            self.assertEqual((False, False), changes[0].executable)
+            if self.wt.supports_setting_file_ids():
+                self.assertEqual((root_id, root_id), changes[0].parent_id)
+                self.assertEqual(b'id-1', changes[0].file_id)
         finally:
             transform.finalize()
 
@@ -1188,78 +1287,120 @@
             old = transform.trans_id_tree_path('old')
             subdir = transform.trans_id_tree_path('subdir')
             new = transform.trans_id_tree_path('new')
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
 
             # content deletion
             transform.delete_contents(old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', None),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform,
+                [InventoryTreeChange(
+                    b'id-1', ('old', 'old'), True, (True, True),
+                    (root_id, root_id),
+                    ('old', 'old'), ('file', None),
+                    (False, False), False)])
 
             # content change
             transform.create_file([b'blah'], old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform,
+                [InventoryTreeChange(
+                    b'id-1', ('old', 'old'), True, (True, True),
+                    (root_id, root_id),
+                    ('old', 'old'), ('file', 'file'),
+                    (False, False), False)])
             transform.cancel_deletion(old)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform,
+                [InventoryTreeChange(
+                    b'id-1', ('old', 'old'), True, (True, True),
+                    (root_id, root_id),
+                    ('old', 'old'), ('file', 'file'),
+                    (False, False), False)])
             transform.cancel_creation(old)
 
             # move file_id to a different file
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             transform.unversion_file(old)
             transform.version_file(new, file_id=b'id-1')
             transform.adjust_path('old', root, new)
-            self.assertEqual([(b'id-1', ('old', 'old'), True, (True, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            if transform._tree.supports_setting_file_ids():
+                self.assertTreeChanges(
+                    transform,
+                    [InventoryTreeChange(
+                        b'id-1', ('old', 'old'), True, (True, True),
+                        (root_id, root_id),
+                        ('old', 'old'), ('file', 'file'),
+                        (False, False), False)])
+            else:
+                self.assertTreeChanges(
+                    transform,
+                    [TreeChange(
+                        (None, 'old'), False, (False, True),
+                        (None, 'old'), (None, 'file'), (False, False), False),
+                     TreeChange(
+                         ('old', None), False, (True, False), ('old', 'old'),
+                         ('file', 'file'), (False, False), False)])
+
             transform.cancel_versioning(new)
             transform._removed_id = set()
 
             # execute bit
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             transform.set_executability(True, old)
-            self.assertEqual([(b'id-1', ('old', 'old'), False, (True, True),
-                               (root_id, root_id),
-                               ('old', 'old'), ('file', 'file'),
-                               (False, True), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform,
+                [InventoryTreeChange(
+                    b'id-1', ('old', 'old'), False, (True, True),
+                    (root_id, root_id),
+                    ('old', 'old'), ('file', 'file'),
+                    (False, True), False)])
             transform.set_executability(None, old)
 
             # filename
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             transform.adjust_path('new', root, old)
             transform._new_parent = {}
-            self.assertEqual([(b'id-1', ('old', 'new'), False, (True, True),
-                               (root_id, root_id),
-                               ('old', 'new'), ('file', 'file'),
-                               (False, False), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform,
+                [InventoryTreeChange(
+                    b'id-1', ('old', 'new'), False, (True, True),
+                    (root_id, root_id),
+                    ('old', 'new'), ('file', 'file'),
+                    (False, False), False)])
             transform._new_name = {}
 
             # parent directory
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             transform.adjust_path('new', subdir, old)
             transform._new_name = {}
-            self.assertEqual([(b'id-1', ('old', 'subdir/old'), False,
-                               (True, True), (root_id, b'subdir-id'), ('old', 'old'),
-                               ('file', 'file'), (False, False), False)],
-                             list(transform.iter_changes()))
+            self.assertTreeChanges(
+                transform, [
+                    InventoryTreeChange(
+                        b'id-1', ('old', 'subdir/old'), False,
+                        (True, True), (root_id, b'subdir-id'), ('old', 'old'),
+                        ('file', 'file'), (False, False), False)])
             transform._new_path = {}
-
         finally:
             transform.finalize()
 
+    def assertTreeChanges(self, tt, expected):
+        # TODO(jelmer): Turn this into a matcher?
+        actual = list(tt.iter_changes())
+        if tt._tree.supports_setting_file_ids():
+            self.assertEqual(expected, actual)
+        else:
+            expected = [
+                TreeChange(path=c.path, changed_content=c.changed_content,
+                           versioned=c.versioned, name=c.name,
+                           kind=c.kind, executable=c.executable,
+                           copied=c.copied) for c in expected]
+            actual = [
+                TreeChange(path=c.path, changed_content=c.changed_content,
+                           versioned=c.versioned, name=c.name,
+                           kind=c.kind, executable=c.executable,
+                           copied=c.copied) for c in actual]
+            self.assertEqual(expected, actual)
+
     def test_iter_changes_modified_bleed(self):
         root_id = self.wt.path2id('')
         """Modified flag should not bleed from one change to another"""
@@ -1273,17 +1414,17 @@
         transform.apply()
         transform, root = self.transform()
         try:
-            transform.delete_contents(transform.trans_id_file_id(b'id-1'))
-            transform.set_executability(True,
-                                        transform.trans_id_file_id(b'id-2'))
-            self.assertEqual(
-                [(b'id-1', (u'file1', u'file1'), True, (True, True),
-                 (root_id, root_id), ('file1', u'file1'),
-                 ('file', None), (False, False), False),
-                 (b'id-2', (u'file2', u'file2'), False, (True, True),
-                 (root_id, root_id), ('file2', u'file2'),
-                 ('file', 'file'), (False, True), False)],
-                list(transform.iter_changes()))
+            transform.delete_contents(transform.trans_id_tree_path('file1'))
+            transform.set_executability(True, transform.trans_id_tree_path('file2'))
+            self.assertTreeChanges(transform, [
+                InventoryTreeChange(
+                b'id-1', (u'file1', u'file1'), True, (True, True),
+                (root_id, root_id), ('file1', u'file1'),
+                ('file', None), (False, False), False),
+                InventoryTreeChange(
+                b'id-2', (u'file2', u'file2'), False, (True, True),
+                (root_id, root_id), ('file2', u'file2'),
+                ('file', 'file'), (False, True), False)])
         finally:
             transform.finalize()
 
@@ -1301,12 +1442,17 @@
         floater = transform.trans_id_tree_path('floater')
         try:
             transform.adjust_path('flitter', root, floater)
-            self.assertEqual([(b'floater-id', ('floater', 'flitter'), False,
-                               (True, True),
-                               (root_id, root_id),
-                               ('floater', 'flitter'),
-                               (None, None), (False, False), False)],
-                             list(transform.iter_changes()))
+            if self.wt.has_versioned_directories():
+                self.assertTreeChanges(
+                    transform,
+                    [InventoryTreeChange(
+                        b'floater-id', ('floater', 'flitter'), False,
+                        (True, True),
+                        (root_id, root_id),
+                        ('floater', 'flitter'),
+                        (None, None), (False, False), False)])
+            else:
+                self.assertTreeChanges(transform, [])
         finally:
             transform.finalize()
 
@@ -1320,14 +1466,14 @@
         try:
             old = transform.trans_id_tree_path('old')
             subdir = transform.trans_id_tree_path('subdir')
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
             transform.delete_contents(subdir)
             transform.create_directory(subdir)
             transform.set_executability(False, old)
             transform.unversion_file(old)
             transform.version_file(old, file_id=b'id-1')
             transform.adjust_path('old', root, old)
-            self.assertEqual([], list(transform.iter_changes()))
+            self.assertTreeChanges(transform, [])
         finally:
             transform.finalize()
 
@@ -1415,7 +1561,7 @@
     def test_noname_contents(self):
         """TreeTransform should permit deferring naming files."""
         transform, root = self.transform()
-        parent = transform.trans_id_file_id(b'parent-id')
+        parent = transform.trans_id_tree_path('parent')
         try:
             transform.create_directory(parent)
         except KeyError:
@@ -1425,7 +1571,7 @@
     def test_noname_contents_nested(self):
         """TreeTransform should permit deferring naming files."""
         transform, root = self.transform()
-        parent = transform.trans_id_file_id(b'parent-id')
+        parent = transform.trans_id_tree_path('parent-early')
         try:
             transform.create_directory(parent)
         except KeyError:
@@ -1650,7 +1796,7 @@
 
     def test_no_final_path(self):
         transform, root = self.transform()
-        trans_id = transform.trans_id_file_id(b'foo')
+        trans_id = transform.trans_id_tree_path('foo')
         transform.create_file([b'bar'], trans_id)
         transform.cancel_creation(trans_id)
         transform.apply()

=== modified file 'breezy/tests/per_workingtree/test_unversion.py'
--- breezy/tests/per_workingtree/test_unversion.py	2020-08-10 03:38:37 +0000
+++ breezy/tests/per_workingtree/test_unversion.py	2020-08-15 17:47:31 +0000
@@ -179,7 +179,7 @@
         if tree_b.has_versioned_directories():
             self.assertEqual(4, num_conflicts)
         else:
-            self.assertEqual(3, num_conflicts)
+            self.assertEqual(1, num_conflicts)
 
         self.assertThat(
             tree_b,

=== modified file 'breezy/tests/per_workingtree/test_workingtree.py'
--- breezy/tests/per_workingtree/test_workingtree.py	2020-08-10 15:00:17 +0000
+++ breezy/tests/per_workingtree/test_workingtree.py	2020-08-22 22:46:24 +0000
@@ -214,13 +214,11 @@
     def test_lock_locks_branch(self):
         tree = self.make_branch_and_tree('.')
         self.assertEqual(None, tree.branch.peek_lock_mode())
-        tree.lock_read()
-        self.assertEqual('r', tree.branch.peek_lock_mode())
-        tree.unlock()
+        with tree.lock_read():
+            self.assertEqual('r', tree.branch.peek_lock_mode())
         self.assertEqual(None, tree.branch.peek_lock_mode())
-        tree.lock_write()
-        self.assertEqual('w', tree.branch.peek_lock_mode())
-        tree.unlock()
+        with tree.lock_write():
+            self.assertEqual('w', tree.branch.peek_lock_mode())
         self.assertEqual(None, tree.branch.peek_lock_mode())
 
     def test_revert(self):
@@ -1011,6 +1009,16 @@
         if tree.branch.repository._format.supports_versioned_directories:
             self.assertEqual('directory', tree.stored_kind('b'))
 
+    def test_stored_kind_nonexistent(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.assertRaises(errors.NoSuchFile, tree.stored_kind, 'a')
+        self.addCleanup(tree.unlock)
+        self.build_tree(['tree/a'])
+        self.assertRaises(errors.NoSuchFile, tree.stored_kind, 'a')
+        tree.add(['a'])
+        self.assertIs('file', tree.stored_kind('a'))
+
     def test_missing_file_sha1(self):
         """If a file is missing, its sha1 should be reported as None."""
         tree = self.make_branch_and_tree('.')

=== modified file 'breezy/tests/test_merge.py'
--- breezy/tests/test_merge.py	2020-08-10 15:00:17 +0000
+++ breezy/tests/test_merge.py	2020-08-22 22:46:24 +0000
@@ -1432,7 +1432,8 @@
                            ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'a', u'a'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_not_in_base(self):
@@ -1478,7 +1479,8 @@
                            ((None, [u'bar', u'bar']), u'bar', u'bar'),
                            ((None, [root_id, root_id]), root_id, root_id),
                            ((None, [u'bar', u'bar']), u'bar', u'bar'),
-                           ((None, [False, False]), False, False)),
+                           ((None, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_not_in_this(self):
@@ -1511,7 +1513,8 @@
                            ((u'a', [u'a', u'a']), u'a', None),
                            ((root_id, [root_id, root_id]), root_id, None),
                            ((u'a', [u'a', u'a']), u'a', None),
-                           ((False, [False, False]), False, None)),
+                           ((False, [False, False]), False, None),
+                           False),
                           ], entries)
 
     def test_file_not_in_one_lca(self):
@@ -1562,7 +1565,8 @@
                            ((u'a', [u'a', u'a']), None, u'a'),
                            ((root_id, [root_id, root_id]), None, root_id),
                            ((u'a', [u'a', u'a']), None, u'a'),
-                           ((False, [False, False]), None, False)),
+                           ((False, [False, False]), None, False),
+                           False),
                           ], entries)
 
     def test_not_in_other_or_lca(self):
@@ -1628,7 +1632,8 @@
                            ((u'foo', [u'foo', None]), None, u'foo'),
                            ((root_id, [root_id, None]), None, root_id),
                            ((u'foo', [u'foo', None]), None, 'foo'),
-                           ((False, [False, None]), None, False)),
+                           ((False, [False, None]), None, False),
+                           False),
                           ], entries)
 
     def test_only_in_one_lca(self):
@@ -1683,7 +1688,8 @@
                            ((None, [None, None]), u'a', None),
                            ((None, [None, None]), root_id, None),
                            ((None, [None, None]), u'a', None),
-                           ((None, [None, None]), False, None)),
+                           ((None, [None, None]), False, None),
+                           False),
                           ], entries)
 
     def test_one_lca_supersedes(self):
@@ -1814,7 +1820,8 @@
                            [(b'foo-id', False,
                              ((root_id, [root_id, root_id]), root_id, root_id),
                                ((u'foo', [u'bar', u'foo']), u'bar', u'bing'),
-                               ((False, [False, False]), False, False)),
+                               ((False, [False, False]), False, False),
+                             False),
                             ], entries)
 
     def test_both_sides_revert(self):
@@ -1846,7 +1853,8 @@
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_different_lca_resolve_one_side_updates_content(self):
@@ -1884,7 +1892,8 @@
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_same_lca_resolution_one_side_updates_content(self):
@@ -1946,7 +1955,8 @@
                            ((u'a', [u'a', u'a']), u'b', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'b', u'a'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_kind_changed(self):
@@ -1972,7 +1982,8 @@
                            ((u'a', [u'a', u'a']), u'a', u'a'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'a']), u'a', u'a'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_this_changed_kind(self):
@@ -2018,7 +2029,8 @@
                            ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'b'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_interesting_file_in_this(self):
@@ -2046,7 +2058,8 @@
                            ((u'b', [u'b', u'b']), u'b', u'c'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'c'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_interesting_file_in_base(self):
@@ -2076,7 +2089,8 @@
                            ((u'c', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'c', [u'b', u'b']), u'b', u'b'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_interesting_file_in_lca(self):
@@ -2104,7 +2118,8 @@
                            ((u'b', [u'c', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'c', u'b']), u'b', u'b'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_interesting_files(self):
@@ -2129,7 +2144,8 @@
                            ((u'b', [u'b', u'b']), u'b', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'b', [u'b', u'b']), u'b', u'b'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
 
@@ -2439,7 +2455,8 @@
                            ((u'foo', [u'barry', u'foo']), u'blah', u'barry'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'barry', u'foo']), u'blah', u'barry'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
         conflicts = wt.merge_from_branch(wt.branch, to_revision=b'F-id')
         self.assertEqual(0, conflicts)
@@ -2544,7 +2561,8 @@
                            ((None, [u'foo', None]), u'foo', u'foo'),
                            ((None, [root_id, None]), root_id, root_id),
                            ((None, [u'foo', None]), u'foo', u'foo'),
-                           ((None, [False, None]), False, False)),
+                           ((None, [False, None]), False, False),
+                           False),
                           ], entries)
 
     def test_symlink_all_wt(self):
@@ -2606,7 +2624,8 @@
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_other_reverted_path_to_base(self):
@@ -2733,12 +2752,14 @@
                            ((u'a', [u'a', u'b']), u'c', u'b'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'a', [u'a', u'b']), u'c', u'b'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           (b'foo-id', True,
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'foo', [u'foo', u'foo']), u'foo', u'foo'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_nested_tree_unmodified(self):
@@ -2853,7 +2874,8 @@
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
     def test_nested_tree_subtree_renamed_and_modified(self):
@@ -2898,7 +2920,8 @@
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
                            ((root_id, [root_id, root_id]), root_id, root_id),
                            ((u'sub', [u'sub', u'sub']), u'alt_sub', u'sub'),
-                           ((False, [False, False]), False, False)),
+                           ((False, [False, False]), False, False),
+                           False),
                           ], entries)
 
 

=== modified file 'breezy/transform.py'
--- breezy/transform.py	2020-08-10 15:00:17 +0000
+++ breezy/transform.py	2020-08-22 22:46:24 +0000
@@ -65,12 +65,10 @@
 class NoFinalPath(BzrError):
 
     _fmt = ("No final name for trans_id %(trans_id)r\n"
-            "file-id: %(file_id)r\n"
             "root trans-id: %(root_trans_id)r\n")
 
     def __init__(self, trans_id, transform):
         self.trans_id = trans_id
-        self.file_id = transform.final_file_id(trans_id)
         self.root_trans_id = transform.root
 
 
@@ -235,7 +233,7 @@
 
     def create_path(self, name, parent):
         """Assign a transaction id to a new path"""
-        trans_id = self._assign_id()
+        trans_id = self.assign_id()
         unique_add(self._new_name, trans_id, name)
         unique_add(self._new_parent, trans_id, parent)
         return trans_id
@@ -244,7 +242,7 @@
         """Change the path that is assigned to a transaction id."""
         if parent is None:
             raise ValueError("Parent trans-id may not be None")
-        if trans_id == self._new_root:
+        if trans_id == self.root:
             raise CantMoveRoot
         self._new_name[trans_id] = name
         self._new_parent[trans_id] = parent
@@ -273,7 +271,7 @@
         """
         raise NotImplementedError(self.fixup_new_roots)
 
-    def _assign_id(self):
+    def assign_id(self):
         """Produce a new tranform id"""
         new_id = "new-%s" % self._id_number
         self._id_number += 1
@@ -283,7 +281,7 @@
         """Determine (and maybe set) the transaction ID for a tree path."""
         path = self.canonical_path(path)
         if path not in self._tree_path_ids:
-            self._tree_path_ids[path] = self._assign_id()
+            self._tree_path_ids[path] = self.assign_id()
             self._tree_id_paths[self._tree_path_ids[path]] = path
         return self._tree_path_ids[path]
 
@@ -350,6 +348,8 @@
             contents insertion command)
         """
         if trans_id in self._new_contents:
+            if trans_id in self._new_reference_revision:
+                return 'tree-reference'
             return self._new_contents[trans_id]
         elif trans_id in self._removed_contents:
             return None
@@ -620,7 +620,7 @@
         self.transform = transform
 
     def _determine_path(self, trans_id):
-        if (trans_id == self.transform.root or trans_id == ROOT_PARENT):
+        if trans_id == self.transform.root or trans_id == ROOT_PARENT:
             return u""
         name = self.transform.final_name(trans_id)
         parent_id = self.transform.final_parent(trans_id)
@@ -1233,7 +1233,11 @@
 def resolve_non_directory_parent(tt, path_tree, c_type, parent_id):
     parent_parent = tt.final_parent(parent_id)
     parent_name = tt.final_name(parent_id)
-    parent_file_id = tt.final_file_id(parent_id)
+    # TODO(jelmer): Make this code transform-specific
+    if tt._tree.supports_setting_file_ids():
+        parent_file_id = tt.final_file_id(parent_id)
+    else:
+        parent_file_id = b'DUMMY'
     new_parent_id = tt.new_directory(parent_name + '.new',
                                      parent_parent, parent_file_id)
     _reparent_transform_children(tt, parent_id, new_parent_id)
@@ -1354,6 +1358,9 @@
         self._all_children_cache = {}
         self._final_name_cache = {}
 
+    def supports_setting_file_ids(self):
+        raise NotImplementedError(self.supports_setting_file_ids)
+
     @property
     def _by_parent(self):
         if self.__by_parent is None:
@@ -1380,6 +1387,11 @@
         pass
 
     def _path2trans_id(self, path):
+        """Look up the trans id associated with a path.
+
+        :param path: path to look up, None when the path does not exist
+        :return: trans_id
+        """
         # We must not use None here, because that is a valid value to store.
         trans_id = self._path2trans_id_cache.get(path, object)
         if trans_id is not object:

=== modified file 'breezy/tree.py'
--- breezy/tree.py	2020-08-10 15:00:17 +0000
+++ breezy/tree.py	2020-08-22 22:46:24 +0000
@@ -152,6 +152,13 @@
             return (self.executable[0] != self.executable[1])
         return False
 
+    @property
+    def renamed(self):
+        return (
+            not self.copied and
+            None not in self.name and
+            self.path[0] != self.path[1])
+
     def is_reparented(self):
         return os.path.dirname(self.path[0]) != os.path.dirname(self.path[1])
 

=== modified file 'breezy/workingtree.py'
--- breezy/workingtree.py	2020-08-10 15:00:17 +0000
+++ breezy/workingtree.py	2020-08-22 22:46:24 +0000
@@ -709,8 +709,12 @@
 
     def get_symlink_target(self, path):
         abspath = self.abspath(path)
-        target = osutils.readlink(abspath)
-        return target
+        try:
+            return osutils.readlink(abspath)
+        except OSError as e:
+            if getattr(e, 'errno', None) == errno.ENOENT:
+                raise errors.NoSuchFile(path)
+            raise
 
     def subsume(self, other_tree):
         raise NotImplementedError(self.subsume)

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2019-09-01 15:33:59 +0000
+++ doc/developers/HACKING.txt	2020-08-13 18:22:05 +0000
@@ -377,7 +377,7 @@
 
 .. _pdb: http://docs.python.org/lib/debugger-commands.html
 
-If the ``BZR_PDB`` environment variable is set
+If the ``BRZ_PDB`` environment variable is set
 then brz will go into pdb post-mortem mode when an unhandled exception
 occurs.
 
@@ -386,7 +386,7 @@
 Unix.  SIGBREAK is generated with Ctrl-Pause on Windows (some laptops have
 this as Fn-Pause).  You can continue execution by typing ``c``.  This can
 be disabled if necessary by setting the environment variable
-``BZR_SIGQUIT_PDB=0``.
+``BRZ_SIGQUIT_PDB=0``.
 
 All tests inheriting from bzrlib.tests.TestCase can use ``self.debug()``
 instead of the longer ``import pdb; pdb.set_trace()``. The former also works

=== modified file 'doc/developers/win32_build_setup.txt'
--- doc/developers/win32_build_setup.txt	2017-11-11 12:51:45 +0000
+++ doc/developers/win32_build_setup.txt	2020-08-13 18:22:05 +0000
@@ -23,8 +23,8 @@
    At present the current version is 1.5.25-15. This is used primarily to
    install the build scripts and gcc-mingw. Note that we explicitly *don't*
    install cygwin's python or Breezy package. As we are only interested in
-   running the native version of bzr. For information on running the cygwin
-   port of bzr, look elsewhere.
+   running the native version of brz. For information on running the cygwin
+   port of brz, look elsewhere.
 
    Probably not all of these packages are necessary, but they make life easier.
 
@@ -120,7 +120,7 @@
    http://www.chiark.greenend.org.uk/~sgtatham/putty/download.html
 
    Note that you probably want to set the environment variable
-   ``BZR_SSH=paramiko`` at this time. Otherwise it will try to use the
+   ``BRZ_SSH=paramiko`` at this time. Otherwise it will try to use the
    ``ssh.exe`` that it finds on your PATH (as configured in step 3), and
    cygwin's openssh does *not* know how to access Pageant.
 
@@ -136,8 +136,8 @@
    them. They seem to be owned by ``Administrator`` rather than by
    ``Administrators``.
 
-6) Install bzr. Usually it is easiest to just get the latest all-in-one
-   installer from https://launchpad.net/bzr/+download
+6) Install brz. Usually it is easiest to just get the latest all-in-one
+   installer from https://launchpad.net/brz/+download
 
 7) Install INNOSetup from:
    http://www.jrsoftware.org/isdl.php

=== modified file 'doc/en/admin-guide/advanced.txt'
--- doc/en/admin-guide/advanced.txt	2019-09-01 15:33:59 +0000
+++ doc/en/admin-guide/advanced.txt	2020-08-13 18:22:05 +0000
@@ -54,17 +54,17 @@
  
   $ cd /srv/bzr/projectx
   $ mv trunk trunk              # can simply rename on the filesystem
-  $ bzr branch trunk texas-integration   # very fast in a shared repository
+  $ brz branch trunk texas-integration   # very fast in a shared repository
 
 In Australia, we need to set up the ``/srv/bzr/projectx`` directory and get a
 copy of the current trunk as australia-integration::
 
   $ mkdir -p /srv/bzr
   $ cd /srv/bzr
-  $ bzr init-shared-repo --no-trees projectx
+  $ brz init-shared-repo --no-trees projectx
   $ cd projectx
-  $ bzr branch bzr+ssh://server.example.com/srv/bzr/trunk
-  $ bzr branch trunk australia-integration
+  $ brz branch bzr+ssh://server.example.com/srv/bzr/trunk
+  $ brz branch trunk australia-integration
 
 Merging to master
 ~~~~~~~~~~~~~~~~~
@@ -79,28 +79,28 @@
   # Get a copy of the Australia branch in Texas.  After the initial branch
   # command, use pull to keep the branch up to date.  With a slow network,
   # this is the only slow part
-  $ bzr branch bzr+ssh://autralia.example.com/srv/bzr/projectx/australia-integration \
+  $ brz branch bzr+ssh://autralia.example.com/srv/bzr/projectx/australia-integration \
     bzr+ssh://server.example.com/srv/bzr/projectx/australia-integration
   
   # Check out the master branch locally for doing the merge
   $ cd ~/projectx
-  $ bzr checkout bzr+ssh://server.example.com/srv/bzr/projectx/trunk
+  $ brz checkout bzr+ssh://server.example.com/srv/bzr/projectx/trunk
   $ cd trunk
-  $ bzr merge bzr+ssh://server.example.com/srv/bzr/projectx/texas-integration
+  $ brz merge bzr+ssh://server.example.com/srv/bzr/projectx/texas-integration
   # Run the test suite and resolve any conflicts
-  $ bzr commit -m "Merge Texas branch to master"
+  $ brz commit -m "Merge Texas branch to master"
   
   # Now, merge from Australia using the local copy of that branch
-  $ bzr merge bzr+ssh://server.example.com/srv/bzr/projectx/australia-integration
+  $ brz merge bzr+ssh://server.example.com/srv/bzr/projectx/australia-integration
   # Run the test suite and resolve any conflicts between the two offices
-  $ bzr commit -m "Merge Australia branch to master"
+  $ brz commit -m "Merge Australia branch to master"
 
 Note that Bazaar does not commit even cleanly applied merges by default.  This
 is because although a merge may apply cleanly, the merged state still needs to
 be checked before it is committed.  (Just because there are no text conflicts
 does not mean that everything will work after a merge.)  An alternative that 
 can pull when possible and merge otherwise is available with 
-``bzr merge --pull``.
+``brz merge --pull``.
 
 Merging back to local trunks
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -108,38 +108,38 @@
 Now the trunk branch is the most up-to-date version of the software and
 both of the local trunks need to reincorporate the changes from the master.
 If no new commits have been made to texas-integration, then that can happen using
-``bzr pull``::
+``brz pull``::
 
   $ cd ~/projectx
-  $ bzr checkout bzr+ssh://server.example.com/srv/bzr/projectx/texas-integration
+  $ brz checkout bzr+ssh://server.example.com/srv/bzr/projectx/texas-integration
   $ cd texas-integration
-  $ bzr pull ../trunk  # Use trunk from the local disk
+  $ brz pull ../trunk  # Use trunk from the local disk
                               # No need to commit
 
 If new changes have happened on texas-integration since the integration with
 trunk, then the above pull will produce an error suggesting to use
 merge::
 
-  $ bzr merge ../trunk
+  $ brz merge ../trunk
   # Run test suite, resolve conflicts
-  $ bzr commit -m "Merging Australian changes"
+  $ brz commit -m "Merging Australian changes"
 
 In Australia, they will need to update their local copy of trunk::
 
   $ cd /srv/bzr/projectx/trunk
-  $ bzr pull     # parent location is used by default
+  $ brz pull     # parent location is used by default
 
 Then, they need to pull or merge the changes from trunk into the local trunk.
 This should be done by a developer with a checkout of australia-integration so
 that they can run the test suite::
 
   $ cd ~/projectx
-  $ bzr co bzr+ssh://australia.example.com/srv/bzr/projectx/australia-integration
+  $ brz co bzr+ssh://australia.example.com/srv/bzr/projectx/australia-integration
   $ cd australia-integration
-  $ bzr merge bzr+ssh://australia.example.com/srv/bzr/projectx/trunk
+  $ brz merge bzr+ssh://australia.example.com/srv/bzr/projectx/trunk
   # Run test suite and integrate Texan changes with only recent local
   # development
-  $ bzr commit -m "Integrate work from Texas"
+  $ brz commit -m "Integrate work from Texas"
 
 
 Other Considerations

=== modified file 'doc/en/admin-guide/backup.txt'
--- doc/en/admin-guide/backup.txt	2018-11-24 15:56:05 +0000
+++ doc/en/admin-guide/backup.txt	2020-08-13 18:22:05 +0000
@@ -44,7 +44,7 @@
 The features that make Bazaar a good distributed version control system also
 make it a good choice for backing itself up.  In particular, complete and
 consistent copies of any branch can easily be obtained with the ``branch`` and
-``pull`` commands.  As a result, a backup process can simply run ``bzr pull``
+``pull`` commands.  As a result, a backup process can simply run ``brz pull``
 on a copy of the main branch to fully update that copy.  If this backup
 process runs periodically, then the backups will be as current as the last
 time that ``pull`` was run.  (This is in addition to the fact
@@ -76,7 +76,7 @@
 Bound Branch Backups
 ~~~~~~~~~~~~~~~~~~~~
 
-When ``bzr pull`` is run regularly to keep a backup copy up to date, then it
+When ``brz pull`` is run regularly to keep a backup copy up to date, then it
 is possible that there are new revisions in the original branch that have not
 yet been pulled into the backup branch.  To alleviate this problem, we can set
 the branches up so that new revisions are *pushed* to the backup rather than
@@ -86,8 +86,8 @@
 is set up on the server itself rather than on the backup machine.  For each
 branch that should be backed up, you just need to use the ``bind`` command to
 set the URL for the backup branch.  In our example, we first need to create
-the branches on the backup server (we'll use ``bzr push``, but we could as
-easily have used ``bzr branch`` from the backup server)
+the branches on the backup server (we'll use ``brz push``, but we could as
+easily have used ``brz branch`` from the backup server)
 
 ::
 
@@ -143,13 +143,13 @@
 but when it came time to restore from backups, finding out that the backups
 themselves were flawed.  As such, it is important to check the quality of the
 backups periodically.  In Bazaar, there are two ways to do this: using the
-``bzr check`` command and by simply making a new branch from the backup.  The
-``bzr check`` command goes through all of the revisions in a branch and checks
+``brz check`` command and by simply making a new branch from the backup.  The
+``brz check`` command goes through all of the revisions in a branch and checks
 them for validity according to Bazaar's internal invariants.  Since it goes
 through every revision, it can be quite slow for large branches.  The other
 way to ensure that the backups can be restored from is to perform a test
 restoration.  This means performing the restoration process in a temporary
-directory.  After the restoration process, ``bzr check`` may again be relevant
+directory.  After the restoration process, ``brz check`` may again be relevant
 for testing the validity of the restored branches.  The following two sections
 present two restoration recipes.
 

=== modified file 'doc/en/admin-guide/hooks-plugins.txt'
--- doc/en/admin-guide/hooks-plugins.txt	2019-02-15 17:41:17 +0000
+++ doc/en/admin-guide/hooks-plugins.txt	2020-08-13 18:22:05 +0000
@@ -8,7 +8,7 @@
 specific tasks are specified by *hooks* that run during certain steps of the
 version control process.  
 
-For full documentation on the available hooks, see ``bzr help hooks``.  Among
+For full documentation on the available hooks, see ``brz help hooks``.  Among
 those, some of the most significant hooks from an administration
 standpoint are `pre_commit`, `post_commit` and `post_change_branch_tip`.
 A `pre_commit` hook can inspect a commit before it happens and cancel it if
@@ -24,10 +24,10 @@
 Information on the whole range of Bazaar plugins is available at
 http://doc.bazaar.canonical.com/plugins/en/.  For purposes of installation,
 plugins are simply python packages.  They can be installed alongside Bazaar in
-the ``bzrlib.plugins`` package using each plugin's ``setup.py``.  They can
+the ``breezy.plugins`` package using each plugin's ``setup.py``.  They can
 also be installed in the plugin path which is the user's
 ``~/.config/breezy/plugins`` directory or can be specified with the
-``BZR_PLUGIN_PATH`` environment variable.  See ``bzr help configuration`` for
+``BRZ_PLUGIN_PATH`` environment variable.  See ``brz help configuration`` for
 more on specifying the location of plugins.
 
 
@@ -55,7 +55,7 @@
 in the ``locations.conf`` file or individually in each branch's
 ``branch.conf`` file.  The sender's email address can be specified as
 ``post_commit_sender`` if it is different than the email address reported by
-``bzr whoami``.  The ``post_commit_mailer`` option specifies how the
+``brz whoami``.  The ``post_commit_mailer`` option specifies how the
 mail should be sent.  If it isn't set, email is sent via ``/usr/bin/mail``.
 It can also be configured to communicate directly with an SMTP server.
 For more details on configuring this plugin, see
@@ -287,7 +287,7 @@
 This plugin is a more advanced version of `checkeol` that can check such
 coding style guidelines such as trailing whitespace, long lines and files that
 don't end with a newline.  It is configured using Bazaar's built in rules
-specification in ``BZR_HOME/rules`` (see ``bzr help rules`` for more
+specification in ``BRZ_HOME/rules`` (see ``bzr help rules`` for more
 information.  For different types of undesired changes, you can specify
 different types of actions.  For example
 

=== modified file 'doc/en/admin-guide/migration.txt'
--- doc/en/admin-guide/migration.txt	2019-02-15 17:41:17 +0000
+++ doc/en/admin-guide/migration.txt	2020-08-13 18:22:05 +0000
@@ -13,14 +13,10 @@
 In many projects wishing to use Bazaar, there is pre-existing history for the
 codebase that should be taken into consideration.  Bazaar leverages an
 interchange format originally developed for Git called `fast-import` to
-provide migration strategies for many other version control systems.  To
-work with fast-import files, Bazaar needs the `fastimport`_ plugin.  This can
-be installed as with any Bazaar plugin.
-
-.. _fastimport: http://launchpad.net/bzr-fastimport
+provide migration strategies for many other version control systems.
 
 The way that fast-import can be used for migration is to export the existing
-history into a fast-import file, then use the ``bzr fast-import`` command.
+history into a fast-import file, then use the ``brz fast-import`` command.
 The `fastimport` plugin includes exporters for Subversion, CVS, Git, Mercurial
 and darcs, accessible as the ``fast-export-from-XXX`` commands.  Note that
 ``fast-import`` should not be used in a branch with existing history.
@@ -66,8 +62,8 @@
 
 This allows substantially complicated Subversion repositories to be converted
 into a set of separate Bazaar branches.  After installing the svn plugin, see
-``bzr help svn-import`` and ``bzr help svn-layout``.
-
-.. _svn: http://launchpad.net/bzr-svn
-
-.. TODO: Legacy VCS to bzr sync.  Tailor?  Incremental conversions?
+``brz help svn-import`` and ``brz help svn-layout``.
+
+.. _svn: http://launchpad.net/brz-svn
+
+.. TODO: Legacy VCS to brz sync.  Tailor?  Incremental conversions?

=== modified file 'doc/en/admin-guide/simple-setups.txt'
--- doc/en/admin-guide/simple-setups.txt	2019-09-01 15:33:59 +0000
+++ doc/en/admin-guide/simple-setups.txt	2020-08-13 18:22:05 +0000
@@ -64,8 +64,8 @@
 have many revisions in common, such as a project trunk and its feature
 branches.
 
-If Bazaar is not installed on the user's path or not specified in the SSH
-configuration, then a path can be specified from the client with the
+If Bazaar or Breezy is not installed on the user's path or not specified in the
+SSH configuration, then a path can be specified from the client with the
 ``BZR_REMOTE_PATH`` environment variable.  For example, if the Bazaar executable
 is installed in ``/usr/local/bzr-2.0/bin/bzr``, then a developer could use::
 
@@ -74,7 +74,7 @@
 
 to get information about the trunk branch.  The remote path can also be
 specified in Bazaar's configuration files for a particular location.  See
-``bzr help configuration`` for more details.
+``brz help configuration`` for more details.
 
 If developers have home directories on the server, they can use ``/~/`` in
 URLs to refer to their home directory.  They can also use ``/~username/`` to
@@ -112,9 +112,9 @@
 
 The following example shows how a single line is configured::
 
-  command="bzr serve --inet --allow-writes --directory=/srv/bzr",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= my bzr key
+  command="brz serve --inet --allow-writes --directory=/srv/bzr",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= my bzr key
 
-This command allows the user to access only bzr and disables other SSH use.  Write
+This command allows the user to access only brz and disables other SSH use.  Write
 access to each repository in the directory ``/srv/bzr`` has been granted with ``--allow-writes``
 and can be removed for individual users that should only require read access.  The root of
 the directory structure can be altered for each user to allow them to see only a subet
@@ -122,14 +122,14 @@
 for Alice and Bob.  This method will not allow you to restrict access to part
 of a repository, you may only restrict access to a single part of the directory structure::
 
-  command="bzr serve --inet --allow-writes --directory=/srv/bzr/alice/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Alice's SSH Key
-  command="bzr serve --inet --allow-writes --directory=/srv/bzr/bob/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Bob's SSH Key
-  command="bzr serve --inet --allow-writes --directory=/srv/bzr/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Repo Manager SSH Key
+  command="brz serve --inet --allow-writes --directory=/srv/bzr/alice/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Alice's SSH Key
+  command="brz serve --inet --allow-writes --directory=/srv/bzr/bob/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Bob's SSH Key
+  command="brz serve --inet --allow-writes --directory=/srv/bzr/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Repo Manager SSH Key
 
 Alice and Bob have access to their own repository and Repo Manager
 has access to the each of their repositories.  Users are not allowed access to any part of 
 the system except the directory specified. The bzr+ssh urls are simplified by 
-serving using ``bzr serve`` and the ``--directory`` option.
+serving using ``brz serve`` and the ``--directory`` option.
 
 If Alice logs in she uses the following command for her fix-1023 branch::
 

=== modified file 'doc/en/upgrade-guide/overview.txt'
--- doc/en/upgrade-guide/overview.txt	2018-11-24 15:56:05 +0000
+++ doc/en/upgrade-guide/overview.txt	2020-08-13 18:22:05 +0000
@@ -63,7 +63,7 @@
 --------------------------
 
 Many plugins are not dependent on a particular Breezy version so
-upgrading them is optional. Other plugins, notably bzr-svn, are more tightly
+upgrading them is optional. Other plugins, notably brz-svn, are more tightly
 associated with Breezy's APIs so these typically need to be upgraded in
 lockstep with the core software.
 

=== modified file 'doc/en/user-guide/configuring_breezy.txt'
--- doc/en/user-guide/configuring_breezy.txt	2017-07-31 00:12:06 +0000
+++ doc/en/user-guide/configuring_breezy.txt	2020-08-13 18:22:05 +0000
@@ -176,7 +176,7 @@
 
 Some commands and plugins provide custom processing on files matching
 certain patterns. Per-user rule-based preferences are defined in
-``BZR_HOME/rules``.
+``BRZ_HOME/rules``.
 
 For further information on how rules are searched and the detailed syntax of
 the relevant files, see `Rules <../user-reference/index.html#rules>`_
@@ -194,4 +194,4 @@
 "words" even if they contain whitespace, and backslash may be used to quote
 quotation marks.  For example::
 
-    BZR_EDITOR="C:\Program Files\My Editor\myeditor.exe"
+    BRZ_EDITOR="C:\Program Files\My Editor\myeditor.exe"

=== modified file 'doc/en/user-guide/plugins.txt'
--- doc/en/user-guide/plugins.txt	2019-02-15 17:41:17 +0000
+++ doc/en/user-guide/plugins.txt	2020-08-13 18:22:05 +0000
@@ -37,7 +37,7 @@
 ``plugins`` directory under your Breezy configuration directory,
 ``~/.config/breezy/`` on Unix and
 ``C:\Documents and Settings\<username>\Application Data\Breezy\2.0\``
-on Windows. Within this directory (referred to as $BZR_HOME below),
+on Windows. Within this directory (referred to as $BRZ_HOME below),
 each plugin is placed in its own subdirectory.
 
 Plugins work particularly well with Breezy branches. For example, to
@@ -50,15 +50,15 @@
 When installing plugins, the directories that you install them in must
 be valid python identifiers. This means that they can only contain
 certain characters, notably they cannot contain hyphens (``-``). Rather
-than installing ``brz-gtk`` to ``$BZR_HOME/plugins/brz-gtk``, install it
-to ``$BZR_HOME/plugins/gtk``.
+than installing ``brz-gtk`` to ``$BRZ_HOME/plugins/brz-gtk``, install it
+to ``$BRZ_HOME/plugins/gtk``.
 
 Alternative plugin locations
 ----------------------------
 
 If you have the necessary permissions, plugins can also be installed on a
 system-wide basis.  One can additionally override the personal plugins
-location by setting the environment variable ``BZR_PLUGIN_PATH`` (see `User
+location by setting the environment variable ``BRZ_PLUGIN_PATH`` (see `User
 Reference <../user-reference/configuration-help.html#brz-plugin-path>`_ 
 for a detailed explanation).
 

=== modified file 'doc/en/user-guide/recording_changes.txt'
--- doc/en/user-guide/recording_changes.txt	2017-07-30 16:59:50 +0000
+++ doc/en/user-guide/recording_changes.txt	2020-08-13 18:22:05 +0000
@@ -29,7 +29,7 @@
 If you use neither the ``-m`` nor the ``-F`` option then brz will open an
 editor for you to enter a message.  The editor to run is controlled by
 your ``$VISUAL`` or ``$EDITOR`` environment variable, which can be overridden
-by the ``editor`` setting in ``~/.config/breezy/breezy.conf``; ``$BZR_EDITOR``
+by the ``editor`` setting in ``~/.config/breezy/breezy.conf``; ``$BRZ_EDITOR``
 will override either of the above mentioned editor options.  If you quit the
 editor without making any changes, the commit will be cancelled.
 

=== modified file 'setup.py'
--- setup.py	2020-07-28 02:11:05 +0000
+++ setup.py	2020-08-22 22:46:24 +0000
@@ -19,8 +19,8 @@
 
 try:
     import setuptools
-except ImportError:
-    sys.stderr.write("[ERROR] Please install setuptools\n")
+except ImportError as e:
+    sys.stderr.write("[ERROR] Please install setuptools (%s)\n" % e)
     sys.exit(1)
 
 

